const db_con = require("../../../config/database-connection");
const generateGLTraGenId = require("../../../Function/gl-transaction-function");
const generateBillNoGenId = require("../../../Function/pl-transaction-function");
const getCurrentDateTime = require("../../../config/datetime.js");

const {
  main_cash_gl_acc_id,
  main_bank_gl_acc_id,
  lab_test_gl_acc_id,
  lab_test_loan_gl_acc_id,
  doctor_ref_fee_gl_acc_id,
  hospital_charge_gl_acc_id,
} = require("../../../config/account-config");

const insertCustomerBillPaymentAdvanceTransaction = async (req, res) => {
  const dateTime = getCurrentDateTime();

  const {
    gl_tra_branch,
    pl_tra_ref_id,
    pl_tra_ref_acc,
    pl_tra_lab_test_discount,
    pl_tra_bill_discount,
    pl_tra_ttl_discount,
    pl_tra_ttl_amount,
    pl_tra_payable_amount,
    pl_tra_cash_bal,
    pl_tra_bank_bal,
    pl_tra_balance,
    pl_tra_user,
    pl_tra_total_payment,
    actual_cash_bal,
    pl_tra_lab_test_tra,
    pltb_collected_time,
    pltb_age,
    pltb_gender,
    pltb_doctorid,
  } = req.body;

  console.log(req.body);

  try {
    // Helper function to generate transaction ID
    const generateGLIdForType = async (type) => {
      return new Promise((resolve, reject) => {
        generateGLTraGenId(db_con, gl_tra_branch, type, (error, id) => {
          if (error) {
            console.error(`Error for type ${type}:`, error);
            reject(error);
          } else {
            resolve(id);
          }
        });
      });
    };

    // Helper function to generate bill ID
    const generateBillIdForType = async (type) => {
      return new Promise((resolve, reject) => {
        generateBillNoGenId(db_con, gl_tra_branch, type, (error, id) => {
          if (error) {
            console.error(`Error for type ${type}:`, error);
            reject(error);
          } else {
            resolve(id);
          }
        });
      });
    };

    let promises = [];
    var status = "";
    var bill_no_print = "";

    // ? set logic for genarate gl tra and pl tra bill and transaction numbers
    if (pl_tra_total_payment >= pl_tra_payable_amount) {
      // If total payment is greater than or equal to payable amount
      status = "PF";
      promises = [
        generateGLIdForType("R"),
        generateGLIdForType("J"),
        generateBillIdForType("PF"),
        generateBillIdForType("PL"),
      ];
    } else if (
      0 < pl_tra_total_payment &&
      pl_tra_total_payment < pl_tra_payable_amount
    ) {
      // If total payment is between 0 and payable amount
      status = "PA";
      promises = [
        generateGLIdForType("R"),
        generateGLIdForType("J"),
        generateBillIdForType("PA"),
        generateBillIdForType("PL"),
      ];
    } else if (pl_tra_total_payment <= 0) {
      // If total payment is less than or equal to 0
      status = "PL";
      promises = [
        "0",
        generateGLIdForType("J"),
        "0",
        generateBillIdForType("PL"),
      ];
    }

    // ? Generate pl and gl transaction IDs for both types
    const [tra_id_R, tra_id_J, bill_no_pa, bill_no_pl] = await Promise.all(
      promises
    );

    // ? update lab test transaction status
    updateLabTestTransactionStatus(pl_tra_lab_test_tra);

    // ? insert loan data to pl tra table
    const query = `
    INSERT INTO pl_tra_tbl 
    (
        pl_tra_lab_test_tra,
        pl_tra_id, 
        pl_tra_bill_no, 
        pl_tra_ref_id, 
        pl_tra_ref_acc, 
        pl_tra_credit,
        pl_tra_lab_test_discount,
        pl_tra_bill_discount,
        pl_tra_ttl_discount, 
        pl_tra_ttl_amount, 
        pl_tra_payable_amount, 
        pl_tra_balance, 
        pl_tra_type, 
        pl_tra_date, 
        pl_tra_time, 
        pl_tra_branch, 
        pl_tra_user
    )
    VALUES (?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?)`;

    const values = [
      pl_tra_lab_test_tra,
      tra_id_J,
      bill_no_pl,
      pl_tra_ref_id,
      pl_tra_ref_acc,
      pl_tra_payable_amount,
      pl_tra_lab_test_discount,
      pl_tra_bill_discount,
      pl_tra_ttl_discount,
      pl_tra_ttl_amount,
      pl_tra_payable_amount,
      0,
      "PL",
      dateTime.date,
      dateTime.time,
      gl_tra_branch,
      pl_tra_user,
    ];

    // ? insert loan recovery data to pl tra table
    const query1 = `
    INSERT INTO pl_tra_tbl
    (
        pl_tra_lab_test_tra,
        pl_tra_id,
        pl_tra_bill_no,
        pl_tra_ref_id,
        pl_tra_ref_acc,
        pl_tra_debit,
        pl_tra_lab_test_discount,
        pl_tra_bill_discount,
        pl_tra_ttl_discount,
        pl_tra_ttl_amount,
        pl_tra_payable_amount,
        pl_tra_cash_bal,
        pl_tra_bank_bal,
        pl_tra_total_payment,
        pl_tra_balance,
        pl_tra_type,
        pl_tra_date,
        pl_tra_time,
        pl_tra_branch,
        pl_tra_user
    )
    VALUES (?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?)`;

    const values1 = [
      pl_tra_lab_test_tra,
      tra_id_R,
      bill_no_pa,
      pl_tra_ref_id,
      pl_tra_ref_acc,
      pl_tra_total_payment,
      pl_tra_lab_test_discount,
      pl_tra_bill_discount,
      pl_tra_ttl_discount,
      pl_tra_ttl_amount,
      pl_tra_payable_amount,
      pl_tra_cash_bal,
      pl_tra_bank_bal,
      pl_tra_cash_bal + pl_tra_bank_bal,
      pl_tra_balance,
      status,
      dateTime.date,
      dateTime.time,
      gl_tra_branch,
      pl_tra_user,
    ];

    // ? gl tra loan data add to debit
    const LoanAccountDebit = {
      gl_tra_id: tra_id_J,
      gl_tra_code: lab_test_loan_gl_acc_id,
      gl_tra_debit: pl_tra_payable_amount,
      gl_tra_credit: 0,
      gl_tra_type: "J",
      gl_tra_status: "Y",
      gl_tra_date: dateTime.date,
      gl_tra_time: dateTime.time,
      gl_tra_description:
        "Lab Test Done For Loan - Transaction. Account Debit.",
      gl_tra_branch: gl_tra_branch,
      gl_tra_user: pl_tra_user,
    };
    insertIntoGLTraTable(LoanAccountDebit);

    // ? gl tra loan data add to credit
    const ServiceAccountCredit = {
      gl_tra_id: tra_id_J,
      gl_tra_code: lab_test_gl_acc_id,
      gl_tra_debit: 0,
      gl_tra_credit: pl_tra_payable_amount,
      gl_tra_type: "J",
      gl_tra_status: "Y",
      gl_tra_date: dateTime.date,
      gl_tra_time: dateTime.time,
      gl_tra_description:
        "Lab Test Done For Loan - Transaction. Account Credit.",
      gl_tra_branch: gl_tra_branch,
      gl_tra_user: pl_tra_user,
    };
    insertIntoGLTraTable(ServiceAccountCredit);

    // ? gl tra loan recovery cash and bank data add
    if (pl_tra_cash_bal > 0) {
      // ? gl tra loan recovery cash balance debit
      const CashDebit = {
        gl_tra_id: tra_id_R,
        gl_tra_code: main_cash_gl_acc_id,
        gl_tra_debit: actual_cash_bal,
        gl_tra_credit: 0,
        gl_tra_type: "R",
        gl_tra_status: "Y",
        gl_tra_date: dateTime.date,
        gl_tra_time: dateTime.time,
        gl_tra_description: "Lab Test Transaction Cash Payment. Account Debit.",
        gl_tra_branch: gl_tra_branch,
        gl_tra_user: pl_tra_user,
      };
      insertIntoGLTraTable(CashDebit);
    }
    // ? gl tra loan recovery cash and bank data add
    if (pl_tra_bank_bal > 0) {
      // ? gl tra loan recovery bank balance debit
      const BankDebit = {
        gl_tra_id: tra_id_R,
        gl_tra_code: main_bank_gl_acc_id,
        gl_tra_debit: pl_tra_bank_bal,
        gl_tra_credit: 0,
        gl_tra_type: "R",
        gl_tra_status: "Y",
        gl_tra_date: dateTime.date,
        gl_tra_time: dateTime.time,
        gl_tra_description: "Lab Test Transaction Bank Payment. Account Debit.",
        gl_tra_branch: gl_tra_branch,
        gl_tra_user: pl_tra_user,
      };
      insertIntoGLTraTable(BankDebit);
    }

    // ? gl tra loan recovery account data credit add
    if (pl_tra_total_payment > 0) {
      const LoanAccountCredit = {
        gl_tra_id: tra_id_R,
        gl_tra_code: lab_test_loan_gl_acc_id,
        gl_tra_debit: 0,
        gl_tra_credit: pl_tra_total_payment,
        gl_tra_type: "R",
        gl_tra_status: "Y",
        gl_tra_date: dateTime.date,
        gl_tra_time: dateTime.time,
        gl_tra_description:
          "Lab Test Loan Recovery Transaction. Account Credit.",
        gl_tra_branch: gl_tra_branch,
        gl_tra_user: pl_tra_user,
      };
      insertIntoGLTraTable(LoanAccountCredit);
    }

    // ? set bill print temp type and bill no
    if (pl_tra_total_payment >= pl_tra_payable_amount) {
      // If total payment is greater than or equal to payable amount
      bill_no_print = bill_no_pa;
    } else if (
      0 < pl_tra_total_payment &&
      pl_tra_total_payment < pl_tra_payable_amount
    ) {
      bill_no_print = bill_no_pa;
    } else if (pl_tra_total_payment <= 0) {
      // If total payment is less than or equal to 0
      bill_no_print = bill_no_pl;
    }

    // ? add data to tem data in print bill table
    const TempBillPrintData = {
      pltb_lab_tra_id: pl_tra_lab_test_tra,
      pltb_collected_time: pltb_collected_time,
      pltb_cus_id: pl_tra_ref_id,
      pltb_cus_acc: pl_tra_ref_acc,
      pltb_age: pltb_age,
      pltb_gender: pltb_gender,
      pltb_doctorid: pltb_doctorid,
      pltb_total_amount: pl_tra_ttl_amount,
      pltb_total_discount: pl_tra_ttl_discount,
      pltb_total_payable: pl_tra_payable_amount,
      pltb_total_payment: pl_tra_total_payment,
      pltb_balance: pl_tra_balance,
      pltb_cash: pl_tra_cash_bal,
      pltb_bank: pl_tra_bank_bal,
      pltb_datetime: dateTime.date + " " + dateTime.time,
      pltb_branch: gl_tra_branch,
      pltb_user: pl_tra_user,
      pltb_bill_no: bill_no_print,
      pltb_bill_type: status,
    };
    insertPrintLabTestBill(TempBillPrintData);

    db_con.query(query, values, (err, results) => {
      if (err) {
        console.error(err);
        return res.status(500).json({ error: "Error inserting data" });
      }

      if (pl_tra_total_payment > 0) {
        // ? If the total payment is greater than 0, execute the query
        db_con.query(query1, values1, (err, results) => {
          if (err) {
            console.error(err);
            return res.status(500).json({ error: "Error inserting data" });
          }

          // Respond with success after the query has been executed successfully
          res.status(200).json({ message: "Successfully added" });
        });
      } else {
        // If total payment is not greater than 0, send an appropriate response
        res.status(200).json({ message: "Successfully added" });
      }
    });
  } catch (error) {
    console.error("Error:", error);
    res.status(500).send(error.message);
  }
};

// ? insert data in due payment collections
const insertDuePaymentCollection = async (req, res) => {
  const dateTime = getCurrentDateTime();

  const {
    gl_tra_branch,
    pl_tra_ref_id,
    pl_tra_ref_acc,
    pl_tra_ttl_amount,
    pl_tra_payable_amount,
    pl_tra_cash_bal,
    pl_tra_bank_bal,
    pl_tra_balance,
    pl_tra_user,
    pl_tra_total_payment,
    actual_cash_bal,
  } = req.body;

  try {
    // Helper function to generate transaction ID
    const generateGLIdForType = async (type) => {
      return new Promise((resolve, reject) => {
        generateGLTraGenId(db_con, gl_tra_branch, type, (error, id) => {
          if (error) {
            console.error(`Error for type ${type}:`, error);
            reject(error);
          } else {
            resolve(id);
          }
        });
      });
    };

    // Helper function to generate bill ID
    const generateBillIdForType = async (type) => {
      return new Promise((resolve, reject) => {
        generateBillNoGenId(db_con, gl_tra_branch, type, (error, id) => {
          if (error) {
            console.error(`Error for type ${type}:`, error);
            reject(error);
          } else {
            resolve(id);
          }
        });
      });
    };

    // ? assign values to variable
    let promises = [];
    var gl_tra_type = "R";
    var pl_tra_type = "PR";

    // ? generate gl transaction id and pl bill no
    promises = [
      generateGLIdForType(gl_tra_type),
      generateBillIdForType(pl_tra_type),
    ];

    // ? Generate pl and gl transaction IDs for both types
    const [tra_id_R, bill_no_pr] = await Promise.all(promises);

    // ? insert loan recovery data to pl tra table
    const query1 = `
     INSERT INTO pl_tra_tbl
     (
         pl_tra_id,
         pl_tra_bill_no,
         pl_tra_ref_id,
         pl_tra_ref_acc,
         pl_tra_debit,
         pl_tra_ttl_amount,
         pl_tra_payable_amount,
         pl_tra_cash_bal,
         pl_tra_bank_bal,
         pl_tra_total_payment,
         pl_tra_balance,
         pl_tra_type,
         pl_tra_date,
         pl_tra_time,
         pl_tra_branch,
         pl_tra_user
     )
     VALUES (?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?)`;

    const values1 = [
      tra_id_R,
      bill_no_pr,
      pl_tra_ref_id,
      pl_tra_ref_acc,
      pl_tra_ttl_amount,
      pl_tra_ttl_amount,
      pl_tra_payable_amount,
      pl_tra_cash_bal,
      pl_tra_bank_bal,
      pl_tra_cash_bal + pl_tra_bank_bal,
      pl_tra_balance,
      pl_tra_type,
      dateTime.date,
      dateTime.time,
      gl_tra_branch,
      pl_tra_user,
    ];

    // ? gl tra loan recovery cash and bank data add
    if (pl_tra_cash_bal > 0) {
      // ? gl tra loan recovery cash balance debit
      const CashDebit = {
        gl_tra_id: tra_id_R,
        gl_tra_code: main_cash_gl_acc_id,
        gl_tra_debit: actual_cash_bal,
        gl_tra_credit: 0,
        gl_tra_type: "R",
        gl_tra_status: "Y",
        gl_tra_date: dateTime.date,
        gl_tra_time: dateTime.time,
        gl_tra_description:
          "Due Payment Collect Transaction Cash Payment. Account Debit.",
        gl_tra_branch: gl_tra_branch,
        gl_tra_user: pl_tra_user,
      };
      insertIntoGLTraTable(CashDebit);
    }

    // ? gl tra loan recovery cash and bank data add
    if (pl_tra_bank_bal > 0) {
      // ? gl tra loan recovery bank balance debit
      const BankDebit = {
        gl_tra_id: tra_id_R,
        gl_tra_code: main_bank_gl_acc_id,
        gl_tra_debit: pl_tra_bank_bal,
        gl_tra_credit: 0,
        gl_tra_type: "R",
        gl_tra_status: "Y",
        gl_tra_date: dateTime.date,
        gl_tra_time: dateTime.time,
        gl_tra_description:
          "Due Payment Collect Transaction Bank Payment. Account Debit.",
        gl_tra_branch: gl_tra_branch,
        gl_tra_user: pl_tra_user,
      };
      insertIntoGLTraTable(BankDebit);
    }

    // ? gl tra loan recovery account data credit add
    if (pl_tra_total_payment > 0) {
      const LoanAccountCredit = {
        gl_tra_id: tra_id_R,
        gl_tra_code: lab_test_loan_gl_acc_id,
        gl_tra_debit: 0,
        gl_tra_credit: pl_tra_ttl_amount,
        gl_tra_type: "R",
        gl_tra_status: "Y",
        gl_tra_date: dateTime.date,
        gl_tra_time: dateTime.time,
        gl_tra_description:
          "Lab Test Loan Recovery Transaction. Account Credit.",
        gl_tra_branch: gl_tra_branch,
        gl_tra_user: pl_tra_user,
      };
      insertIntoGLTraTable(LoanAccountCredit);
    }

    // ? add data to tem data in print bill table
    const TempBillPrintData = {
      pltb_lab_tra_id: "",
      pltb_collected_time: "0000-00-00 00:00:00",
      pltb_cus_id: pl_tra_ref_id,
      pltb_cus_acc: pl_tra_ref_acc,
      pltb_age: ".",
      pltb_gender: ".",
      pltb_doctorid: ".",
      pltb_total_amount: pl_tra_ttl_amount,
      pltb_total_discount: "0",
      pltb_total_payable: pl_tra_payable_amount,
      pltb_total_payment: pl_tra_total_payment,
      pltb_balance: pl_tra_balance,
      pltb_cash: pl_tra_cash_bal,
      pltb_bank: pl_tra_bank_bal,
      pltb_datetime: dateTime.date + " " + dateTime.time,
      pltb_branch: gl_tra_branch,
      pltb_user: pl_tra_user,
      pltb_bill_no: bill_no_pr,
      pltb_bill_type: pl_tra_type,
    };
    insertPrintLabTestBill(TempBillPrintData);

    // ? If the total payment is greater than 0, execute the query
    db_con.query(query1, values1, (err, results) => {
      if (err) {
        console.error(err);
        return res.status(500).json({ error: "Error inserting data" });
      }

      // Respond with success after the query has been executed successfully
      res.status(200).json({ message: "Successfully added" });
    });
  } catch (error) {
    console.error("Error:", error);
    res.status(500).send(error.message);
  }
};

// ? insert bill data to gl tra table
function insertIntoGLTraTable(data) {
  const query = `
      INSERT INTO gl_tra_tbl (
          gl_tra_id,
          gl_tra_code,
          gl_tra_debit,
          gl_tra_credit,
          gl_tra_type,
          gl_tra_status,
          gl_tra_date,
          gl_tra_time,
          gl_tra_description,
          gl_tra_branch,
          gl_tra_user
      )
      VALUES (?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?)`;

  const values = [
    data.gl_tra_id,
    data.gl_tra_code,
    data.gl_tra_debit,
    data.gl_tra_credit,
    data.gl_tra_type,
    data.gl_tra_status,
    data.gl_tra_date,
    data.gl_tra_time,
    data.gl_tra_description,
    data.gl_tra_branch,
    data.gl_tra_user,
  ];

  return new Promise((resolve, reject) => {
    db_con.query(query, values, (error, results) => {
      if (error) {
        reject(error);
      } else {
        resolve(results);
      }
    });
  });
}

// ? update lab test transaction status
function updateLabTestTransactionStatus(transactionID) {
  const query = `
    UPDATE lab_test_tra_temp_tbl
    SET lab_tra_status_tmp = ?
    WHERE lab_tra_id_tmp = ?`;

  const values = ["Y", transactionID];

  return new Promise((resolve, reject) => {
    db_con.query(query, values, (error, results) => {
      if (error) {
        reject(error);
      } else {
        resolve(results);
      }
    });
  });
}

// ? insert to label test print bill table
function insertPrintLabTestBill(data) {
  // Insert query
  const queryInsert = `
    INSERT INTO print_lab_test_bill_tbl 
      (pltb_lab_tra_id,pltb_collected_time, pltb_cus_id, pltb_cus_acc, pltb_age, pltb_gender, pltb_doctorid,
       pltb_total_amount, pltb_total_discount, pltb_total_payable, pltb_total_payment, 
       pltb_balance, pltb_cash, pltb_bank, pltb_datetime, pltb_branch, pltb_user,pltb_bill_no,pltb_bill_type) 
    VALUES (?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?,?,?)
  `;

  const values = [
    data.pltb_lab_tra_id,
    data.pltb_collected_time,
    data.pltb_cus_id,
    data.pltb_cus_acc,
    data.pltb_age,
    data.pltb_gender,
    data.pltb_doctorid,
    data.pltb_total_amount,
    data.pltb_total_discount,
    data.pltb_total_payable,
    data.pltb_total_payment,
    data.pltb_balance,
    data.pltb_cash,
    data.pltb_bank,
    data.pltb_datetime,
    data.pltb_branch,
    data.pltb_user,
    data.pltb_bill_no,
    data.pltb_bill_type,
  ];

  return new Promise((resolve, reject) => {
    db_con.query(queryInsert, values, (error, results) => {
      if (error) {
        reject(error);
      } else {
        resolve(results.insertId);
      }
    });
  });
}

// ? get all re print bill transaction data for table view by patient id
const getAllLabTestTempBillDataTable = async (req, res) => {
  try {
    const { draw, start, length, search, order, columns } = req.query;
    const cusID = req.params.id; // Extracting billNo from parameters
    const page = Math.ceil(start / length) + 1;
    const limit = +length;
    const offset = (page - 1) * limit;

    const orderColumnIndex = order[0].column;
    const orderDirection = order[0].dir;
    const orderColumn = columns[orderColumnIndex].data;

    const searchPattern = `%${search.value}%`;
    let searchWhere = "";
    const searchParams = [];

    if (search.value) {
      searchWhere = " WHERE ";
      // Add your search conditions here based on table columns
      searchParams.push(searchPattern);
    }

    // Adding filter for specific billNo value
    if (cusID) {
      searchWhere += searchWhere ? " AND " : " WHERE ";
      searchWhere += " pltb_cus_id = ? ";
      searchParams.push(cusID);
    }

    const selectQuery = `
      SELECT id,
      pltb_lab_tra_id, 
      pltb_collected_time, 
      pltb_cus_id, 
      pltb_cus_acc, 
      pltb_age, 
      pltb_gender, 
      pltb_doctorid, 
      pltb_total_amount, 
      pltb_total_discount, 
      pltb_total_payable, 
      pltb_total_payment, 
      pltb_balance, 
      pltb_cash, 
      pltb_bank, 
      DATE_FORMAT(pltb_datetime, '%Y-%m-%d %H:%i:%s') AS pltb_datetime_formatted, 
      pltb_branch, 
      pltb_user, 
      pltb_bill_no, 
      pltb_bill_type
      FROM print_lab_test_bill_tbl
      ${searchWhere}
      ORDER BY ${orderColumn} ${orderDirection}
      LIMIT ?, ?
    `;

    const countQuery = `
      SELECT COUNT(pltb_lab_tra_id) AS total 
      FROM print_lab_test_bill_tbl
      ${searchWhere}
    `;

    db_con.query(
      selectQuery,
      [...searchParams, offset, limit],
      (error, results) => {
        if (error) {
          return res.status(500).send(error);
        }

        if (results.length === 0) {
          return res.status(200).json({
            draw: +draw,
            recordsTotal: 0,
            recordsFiltered: 0,
            data: [],
            message: "No data found in the table.",
          });
        }

        db_con.query(countQuery, searchParams, (countError, countResults) => {
          if (countError) {
            return res.status(500).send(countError);
          }

          const totalRecords = countResults[0].total;
          const response = {
            draw: +draw,
            recordsTotal: totalRecords,
            recordsFiltered: totalRecords,
            data: results,
          };

          res.status(200).json(response);
        });
      }
    );
  } catch (error) {
    res.status(500).send(error.message);
  }
};

// ? insert other income transaction to data table
const insertOtherIncomeTransaction = async (req, res) => {
  const dateTime = getCurrentDateTime();

  const {
    gl_tra_branch,
    amount,
    cash_bal,
    bank_bal,
    post_date,
    cheque_no,
    description,
    user_id,
    gl_tra_code,
  } = req.body;

  try {
    // Helper function to generate transaction ID
    const generateGLIdForType = async (type) => {
      return new Promise((resolve, reject) => {
        generateGLTraGenId(db_con, gl_tra_branch, type, (error, id) => {
          if (error) {
            console.error(`Error for type ${type}:`, error);
            reject(error);
          } else {
            resolve(id);
          }
        });
      });
    };

    let promises = [];
    var status = "R";

    promises = [generateGLIdForType(status)];

    // ? Generate pl and gl transaction IDs for both types
    const [tra_id_R] = await Promise.all(promises);

    // ? gl tra loan data add to credit
    const ServiceAccountCredit = {
      gl_tra_id: tra_id_R,
      gl_tra_code: gl_tra_code,
      gl_tra_debit: 0,
      gl_tra_credit: amount,
      gl_tra_type: status,
      gl_tra_status: "Y",
      gl_tra_date: dateTime.date,
      gl_tra_time: dateTime.time,
      gl_tra_description:
        "Other Income Transaction. " + cheque_no + " " + description,
      gl_tra_branch: gl_tra_branch,
      gl_tra_user: user_id,
    };
    insertIntoGLTraTable(ServiceAccountCredit);

    // ? gl tra loan recovery cash and bank data add
    if (cash_bal > 0) {
      // ? gl tra loan recovery cash balance debit
      const CashDebit = {
        gl_tra_id: tra_id_R,
        gl_tra_code: main_cash_gl_acc_id,
        gl_tra_debit: cash_bal,
        gl_tra_credit: 0,
        gl_tra_type: status,
        gl_tra_status: "Y",
        gl_tra_date: dateTime.date,
        gl_tra_time: dateTime.time,
        gl_tra_description:
          "Other Income Transaction. " + cheque_no + " " + description,
        gl_tra_branch: gl_tra_branch,
        gl_tra_user: user_id,
      };
      insertIntoGLTraTable(CashDebit);
    }

    // ? gl tra loan recovery cash and bank data add
    if (bank_bal > 0) {
      // ? gl tra loan recovery bank balance debit
      const BankDebit = {
        gl_tra_id: tra_id_R,
        gl_tra_code: main_bank_gl_acc_id,
        gl_tra_debit: bank_bal,
        gl_tra_credit: 0,
        gl_tra_type: status,
        gl_tra_status: "Y",
        gl_tra_date: dateTime.date,
        gl_tra_time: dateTime.time,
        gl_tra_description:
          "Other Income Transaction. " + cheque_no + " " + description,
        gl_tra_branch: gl_tra_branch,
        gl_tra_user: user_id,
      };
      insertIntoGLTraTable(BankDebit);

      const ChequeData = {
        tra_id: tra_id_R,
        tra_type: status,
        credit_bal: bank_bal,
        debit_bal: 0,
        issue_date: dateTime.date,
        post_date: post_date,
        cheque_no: cheque_no,
        description:
          "Other Expenses Transaction. " + cheque_no + " " + description,
        user_id: user_id,
      };

      insertIntoChequeDataTable(ChequeData);
    }

    // Send success response
    res.status(200).send("Other Income Transaction successfully added.");
  } catch (error) {
    console.error("Error:", error);
    res.status(500).send(error.message);
  }
};

// ? insert other expense transaction to data table
const insertOtherExpenseTransaction = async (req, res) => {
  const dateTime = getCurrentDateTime();

  const {
    gl_tra_branch,
    amount,
    cash_bal,
    bank_bal,
    post_date,
    cheque_no,
    description,
    user_id,
    gl_tra_code,
  } = req.body;

  try {
    // Helper function to generate transaction ID
    const generateGLIdForType = async (type) => {
      return new Promise((resolve, reject) => {
        generateGLTraGenId(db_con, gl_tra_branch, type, (error, id) => {
          if (error) {
            console.error(`Error for type ${type}:`, error);
            reject(error);
          } else {
            resolve(id);
          }
        });
      });
    };

    let promises = [];
    var status = "V";

    promises = [generateGLIdForType(status)];

    // ? Generate pl and gl transaction IDs for both types
    const [tra_id_R] = await Promise.all(promises);

    // ? gl tra loan data add to credit
    const ServiceAccountCredit = {
      gl_tra_id: tra_id_R,
      gl_tra_code: gl_tra_code,
      gl_tra_debit: amount,
      gl_tra_credit: 0,
      gl_tra_type: status,
      gl_tra_status: "Y",
      gl_tra_date: dateTime.date,
      gl_tra_time: dateTime.time,
      gl_tra_description:
        "Other Expenses Transaction. " + cheque_no + " " + description,
      gl_tra_branch: gl_tra_branch,
      gl_tra_user: user_id,
    };
    insertIntoGLTraTable(ServiceAccountCredit);

    // ? gl tra loan recovery cash and bank data add
    if (cash_bal > 0) {
      // ? gl tra loan recovery cash balance debit
      const CashDebit = {
        gl_tra_id: tra_id_R,
        gl_tra_code: main_cash_gl_acc_id,
        gl_tra_debit: 0,
        gl_tra_credit: cash_bal,
        gl_tra_type: status,
        gl_tra_status: "Y",
        gl_tra_date: dateTime.date,
        gl_tra_time: dateTime.time,
        gl_tra_description:
          "Other Expenses Transaction. " + cheque_no + " " + description,
        gl_tra_branch: gl_tra_branch,
        gl_tra_user: user_id,
      };
      insertIntoGLTraTable(CashDebit);
    }

    // ? gl tra loan recovery cash and bank data add
    if (bank_bal > 0) {
      // ? gl tra loan recovery bank balance debit
      const BankDebit = {
        gl_tra_id: tra_id_R,
        gl_tra_code: main_bank_gl_acc_id,
        gl_tra_debit: 0,
        gl_tra_credit: bank_bal,
        gl_tra_type: status,
        gl_tra_status: "Y",
        gl_tra_date: dateTime.date,
        gl_tra_time: dateTime.time,
        gl_tra_description:
          "Other Expenses Transaction. " + cheque_no + " " + description,
        gl_tra_branch: gl_tra_branch,
        gl_tra_user: user_id,
      };
      insertIntoGLTraTable(BankDebit);

      const ChequeData = {
        tra_id: tra_id_R,
        tra_type: status,
        credit_bal: 0,
        debit_bal: bank_bal,
        issue_date: dateTime.date,
        post_date: post_date,
        cheque_no: cheque_no,
        description:
          "Other Expenses Transaction. " + cheque_no + " " + description,
        user_id: user_id,
      };

      insertIntoChequeDataTable(ChequeData);
    }

    // Send success response
    res.status(200).send("Other Expense Transaction successfully added.");
  } catch (error) {
    console.error("Error:", error);
    res.status(500).send(error.message);
  }
};

// ? insert cheque transaction data to table
function insertIntoChequeDataTable(data) {
  const query = `
      INSERT INTO gl_cheque_data_tbl (
          tra_id,
          tra_type,
          credit_bal,
          debit_bal,
          issue_date,
          post_date,
          cheque_no,
          description,
          user_id
      )
      VALUES (?, ?, ?, ?, ?, ?, ?, ?, ?)`;

  const values = [
    data.tra_id,
    data.tra_type,
    data.credit_bal,
    data.debit_bal,
    data.issue_date,
    data.post_date,
    data.cheque_no,
    data.description,
    data.user_id,
  ];

  return new Promise((resolve, reject) => {
    db_con.query(query, values, (error, results) => {
      if (error) {
        reject(error);
      } else {
        resolve(results);
      }
    });
  });
}

// ? insert doctor reference fee payment transaction
const insertDoctorRefFeeTransaction = async (req, res) => {
  const dateTime = getCurrentDateTime();

  const {
    ids,
    pl_tra_ref_id,
    pl_tra_ref_acc,
    gl_tra_branch,
    amount,
    cash_bal,
    bank_bal,
    post_date,
    cheque_no,
    description,
    user_id,
  } = req.body;

  try {
    // Helper function to generate transaction ID
    const generateGLIdForType = async (type) => {
      return new Promise((resolve, reject) => {
        generateGLTraGenId(db_con, gl_tra_branch, type, (error, id) => {
          if (error) {
            console.error(`Error for type ${type}:`, error);
            reject(error);
          } else {
            resolve(id);
          }
        });
      });
    };

    // Helper function to generate bill ID
    const generateBillIdForType = async (type) => {
      return new Promise((resolve, reject) => {
        generateBillNoGenId(db_con, gl_tra_branch, type, (error, id) => {
          if (error) {
            console.error(`Error for type ${type}:`, error);
            reject(error);
          } else {
            resolve(id);
          }
        });
      });
    };

    let promises = [];
    var status = "V";
    var bill_type = "DP";

    promises = [generateGLIdForType(status), generateBillIdForType(bill_type)];

    // ? Generate pl and gl transaction IDs for both types
    const [tra_id_R, bill_no_dp] = await Promise.all(promises);

    // ? insert loan recovery data to pl tra table
    const query1 = `
        INSERT INTO pl_tra_tbl
        (
            pl_tra_id,
            pl_tra_bill_no,
            pl_tra_ref_id,
            pl_tra_ref_acc,
            pl_tra_debit,
            pl_tra_ttl_discount,
            pl_tra_ttl_amount,
            pl_tra_payable_amount,
            pl_tra_cash_bal,
            pl_tra_bank_bal,
            pl_tra_total_payment,
            pl_tra_balance,
            pl_tra_type,
            pl_tra_date,
            pl_tra_time,
            pl_tra_branch,
            pl_tra_user,
            pl_tra_description
        )
        VALUES (?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?)`;

    const values1 = [
      tra_id_R,
      bill_no_dp,
      pl_tra_ref_id,
      pl_tra_ref_acc,
      amount,
      0,
      amount,
      amount,
      cash_bal,
      bank_bal,
      bank_bal + bank_bal,
      0,
      bill_type,
      dateTime.date,
      dateTime.time,
      gl_tra_branch,
      user_id,
      ids,
    ];

    // ? gl tra loan data add to credit
    const ServiceAccountCredit = {
      gl_tra_id: tra_id_R,
      gl_tra_code: doctor_ref_fee_gl_acc_id,
      gl_tra_debit: amount,
      gl_tra_credit: 0,
      gl_tra_type: status,
      gl_tra_status: "Y",
      gl_tra_date: dateTime.date,
      gl_tra_time: dateTime.time,
      gl_tra_description:
        "Doctor Reference Fee Payment. " + cheque_no + " " + description,
      gl_tra_branch: gl_tra_branch,
      gl_tra_user: user_id,
    };
    insertIntoGLTraTable(ServiceAccountCredit);

    // ? gl tra loan recovery cash and bank data add
    if (cash_bal > 0) {
      // ? gl tra loan recovery cash balance debit
      const CashDebit = {
        gl_tra_id: tra_id_R,
        gl_tra_code: main_cash_gl_acc_id,
        gl_tra_debit: 0,
        gl_tra_credit: cash_bal,
        gl_tra_type: status,
        gl_tra_status: "Y",
        gl_tra_date: dateTime.date,
        gl_tra_time: dateTime.time,
        gl_tra_description:
          "Doctor Reference Fee Payment. " + cheque_no + " " + description,
        gl_tra_branch: gl_tra_branch,
        gl_tra_user: user_id,
      };
      insertIntoGLTraTable(CashDebit);
    }

    // ? gl tra loan recovery cash and bank data add
    if (bank_bal > 0) {
      // ? gl tra loan recovery bank balance debit
      const BankDebit = {
        gl_tra_id: tra_id_R,
        gl_tra_code: main_bank_gl_acc_id,
        gl_tra_debit: 0,
        gl_tra_credit: bank_bal,
        gl_tra_type: status,
        gl_tra_status: "Y",
        gl_tra_date: dateTime.date,
        gl_tra_time: dateTime.time,
        gl_tra_description:
          "Doctor Reference Fee Payment. " + cheque_no + " " + description,
        gl_tra_branch: gl_tra_branch,
        gl_tra_user: user_id,
      };
      insertIntoGLTraTable(BankDebit);

      const ChequeData = {
        tra_id: tra_id_R,
        tra_type: status,
        credit_bal: 0,
        debit_bal: bank_bal,
        issue_date: dateTime.date + " " + dateTime.time,
        post_date: post_date,
        cheque_no: cheque_no,
        description:
          "Doctor Reference Fee Payment. " + cheque_no + " " + description,
        user_id: user_id,
      };

      insertIntoChequeDataTable(ChequeData);
    }

    // ? Check if the array of IDs is provided and not empty
    if (!ids || typeof ids !== "string" || ids.trim() === "") {
      return res
        .status(400)
        .json({ message: "Invalid or empty array of IDs provided" });
    }

    // ? Convert the string of IDs to an array
    const idArray = ids.split(",").map((id) => parseInt(id.trim(), 10));

    // ? Check if the array of IDs is valid
    if (!idArray || !Array.isArray(idArray) || idArray.length === 0) {
      return res
        .status(400)
        .json({ message: "Invalid or empty array of IDs provided" });
    }

    // ? Create a placeholder for the '?' in the SQL query based on the number of IDs
    const placeholders = idArray.map(() => "?").join(",");

    const [results] = await db_con.promise().execute(query1, values1);

    // ? update lab test tra table doctor ref fee update status
    const lab_tra_update_query = `
      UPDATE lab_test_tra_tbl
      SET lab_tra_ref_fee_update = 'Y', lab_tra_ref_fee_pay_date = ?
      WHERE id IN (${placeholders});`;

    await db_con
      .promise()
      .execute(lab_tra_update_query, [
        dateTime.date + " " + dateTime.time,
        ...idArray,
      ]);

    // Respond with success after the queries have been executed successfully
    res.status(200).json({ message: "Doctor reference successfully added" });
  } catch (error) {
    console.error("Error:", error);
    res.status(500).send(error.message);
  }
};

// ? insert hospital fee expense transaction to data table
const insertHospitalFeeExpenseTransaction = async (req, res) => {
  const dateTime = getCurrentDateTime();

  const {
    ids,
    gl_tra_branch,
    amount,
    cash_bal,
    bank_bal,
    post_date,
    cheque_no,
    description,
    user_id,
  } = req.body;

  try {
    // Helper function to generate transaction ID
    const generateGLIdForType = async (type) => {
      return new Promise((resolve, reject) => {
        generateGLTraGenId(db_con, gl_tra_branch, type, (error, id) => {
          if (error) {
            console.error(`Error for type ${type}:`, error);
            reject(error);
          } else {
            resolve(id);
          }
        });
      });
    };

    let promises = [];
    var status = "V";

    promises = [generateGLIdForType(status)];

    // ? Generate pl and gl transaction IDs for both types
    const [tra_id_R] = await Promise.all(promises);

    // ? gl tra loan data add to credit
    const ServiceAccountCredit = {
      gl_tra_id: tra_id_R,
      gl_tra_code: hospital_charge_gl_acc_id,
      gl_tra_debit: amount,
      gl_tra_credit: 0,
      gl_tra_type: status,
      gl_tra_status: "Y",
      gl_tra_date: dateTime.date,
      gl_tra_time: dateTime.time,
      gl_tra_description:
        "Hospital Fee Expenses Transaction. " + cheque_no + " " + description,
      gl_tra_branch: gl_tra_branch,
      gl_tra_user: user_id,
    };
    insertIntoGLTraTable(ServiceAccountCredit);

    // ? gl tra loan recovery cash and bank data add
    if (cash_bal > 0) {
      // ? gl tra loan recovery cash balance debit
      const CashDebit = {
        gl_tra_id: tra_id_R,
        gl_tra_code: main_cash_gl_acc_id,
        gl_tra_debit: 0,
        gl_tra_credit: cash_bal,
        gl_tra_type: status,
        gl_tra_status: "Y",
        gl_tra_date: dateTime.date,
        gl_tra_time: dateTime.time,
        gl_tra_description:
          "Hospital Fee Expenses Transaction. " + cheque_no + " " + description,
        gl_tra_branch: gl_tra_branch,
        gl_tra_user: user_id,
      };
      insertIntoGLTraTable(CashDebit);
    }

    // ? gl tra loan recovery cash and bank data add
    if (bank_bal > 0) {
      // ? gl tra loan recovery bank balance debit
      const BankDebit = {
        gl_tra_id: tra_id_R,
        gl_tra_code: main_bank_gl_acc_id,
        gl_tra_debit: 0,
        gl_tra_credit: bank_bal,
        gl_tra_type: status,
        gl_tra_status: "Y",
        gl_tra_date: dateTime.date,
        gl_tra_time: dateTime.time,
        gl_tra_description:
          "Hospital Fee Expenses Transaction. " + cheque_no + " " + description,
        gl_tra_branch: gl_tra_branch,
        gl_tra_user: user_id,
      };
      insertIntoGLTraTable(BankDebit);

      const ChequeData = {
        tra_id: tra_id_R,
        tra_type: status,
        credit_bal: 0,
        debit_bal: bank_bal,
        issue_date: dateTime.date,
        post_date: post_date,
        cheque_no: cheque_no,
        description:
          "Hospital Fee Expenses Transaction. " + cheque_no + " " + description,
        user_id: user_id,
      };

      insertIntoChequeDataTable(ChequeData);
    }

    // ? Check if the array of IDs is provided and not empty
    if (!ids || typeof ids !== "string" || ids.trim() === "") {
      return res
        .status(400)
        .json({ message: "Invalid or empty array of IDs provided" });
    }

    // ? Convert the string of IDs to an array
    const idArray = ids.split(",").map((id) => parseInt(id.trim(), 10));

    // ? Check if the array of IDs is valid
    if (!idArray || !Array.isArray(idArray) || idArray.length === 0) {
      return res
        .status(400)
        .json({ message: "Invalid or empty array of IDs provided" });
    }

    // ? Create a placeholder for the '?' in the SQL query based on the number of IDs
    const placeholders = idArray.map(() => "?").join(",");

    // ? update lab test tra table doctor ref fee update status
    const lab_tra_update_query = `
      UPDATE lab_test_tra_tbl
      SET lab_tra_lab_test_send = 'Y', lab_tra_hospital_fee_update = 'Y', lab_tra_hospital_fee_pay_tra = ?, lab_tra_hospital_fee_pay_date = ?, lab_tra_lab_test_send_time = ?
      WHERE id IN (${placeholders});`;

    await db_con
      .promise()
      .execute(lab_tra_update_query, [
        tra_id_R,
        dateTime.date + " " + dateTime.time,
        dateTime.date + " " + dateTime.time,
        ...idArray,
      ]);

    // Send success response
    res.status(200).send("Hospital Fee Transaction successfully added.");
  } catch (error) {
    console.error("Error:", error);
    res.status(500).send(error.message);
  }
};

module.exports = {
  insertCustomerBillPaymentAdvanceTransaction,
  getAllLabTestTempBillDataTable,
  insertDuePaymentCollection,
  insertOtherIncomeTransaction,
  insertOtherExpenseTransaction,
  insertDoctorRefFeeTransaction,
  insertHospitalFeeExpenseTransaction,
};
