const db_con = require("../../../config/database-connection");

// ! get all pl tra bill transaction data for table view
const getAllPLTraTransactionByPatientId = async (req, res) => {
  try {
    const { draw, start, length, search, order, columns } = req.query;
    const patient_id = req.params.id; // Extracting patient_id from parameters
    const page = Math.ceil(start / length) + 1;
    const limit = +length;
    const offset = (page - 1) * limit;

    const orderColumnIndex = order[0].column;
    const orderDirection = order[0].dir;
    const orderColumn = columns[orderColumnIndex].data;

    const searchPattern = `%${search.value}%`;
    let searchWhere = "";
    const searchParams = [];

    if (search.value) {
      searchWhere = " WHERE ";
      searchWhere += " (pl_tra_credit LIKE ? ";
      searchWhere += " OR pl_tra_debit LIKE ? ";
      searchWhere += " OR pl_tra_date LIKE ? ) ";
      searchParams.push(searchPattern, searchPattern, searchPattern);
    }

    // Adding filter for specific lab_tra_id_tmp value (B01-LT00000001)
    if (patient_id) {
      searchWhere += searchWhere ? " AND " : " WHERE ";
      searchWhere += " pl_tra_ref_id = ? ";
      searchParams.push(patient_id);
    }

    const selectQuery = `
        SELECT 
        pl_tra_lab_test_tra,
        pl_tra_bill_no,
        pl_tra_credit,
        pl_tra_debit,
        pl_tra_ttl_discount,
        pl_tra_ttl_amount,
        pl_tra_payable_amount,
        pl_tra_cash_bal,
        pl_tra_bank_bal,
        pl_tra_total_payment,
        pl_tra_balance,
        DATE_FORMAT(CONCAT(pl_tra_date, ' ', pl_tra_time), '%Y-%m-%d %H:%i:%s') as pl_tra_date,
        pl_tra_type,
        user_fname,
        brach_name
        FROM pl_tra_tbl 
        INNER JOIN sys_user_tbl
        ON 
          pl_tra_tbl.pl_tra_user = sys_user_tbl.user_id
        INNER JOIN sys_branch_tbl
        ON 
          pl_tra_tbl.pl_tra_branch = sys_branch_tbl.brach_id
        ${searchWhere}
        ORDER BY ${orderColumn} ${orderDirection}
        LIMIT ?, ?
      `;

    const countQuery = `
        SELECT COUNT(pl_tra_lab_test_tra) AS total 
        FROM pl_tra_tbl 
        INNER JOIN sys_user_tbl
        ON 
          pl_tra_tbl.pl_tra_user = sys_user_tbl.user_id
        INNER JOIN sys_branch_tbl
        ON 
          pl_tra_tbl.pl_tra_branch = sys_branch_tbl.brach_id
        ${searchWhere}
      `;

    db_con.query(
      selectQuery,
      [...searchParams, offset, limit],
      (error, results) => {
        if (error) {
          return res.status(500).send(error);
        }

        if (results.length === 0) {
          return res.status(200).json({
            draw: +draw,
            recordsTotal: 0,
            recordsFiltered: 0,
            data: [],
            message: "No data found in the table.",
          });
        }

        db_con.query(countQuery, searchParams, (countError, countResults) => {
          if (countError) {
            return res.status(500).send(countError);
          }

          const totalRecords = countResults[0].total;
          const response = {
            draw: +draw,
            recordsTotal: totalRecords,
            recordsFiltered: totalRecords,
            data: results,
          };

          res.status(200).json(response);
        });
      }
    );
  } catch (error) {
    res.status(500).send(error.message);
  }
};

module.exports = {
  getAllPLTraTransactionByPatientId,
};
