const db_con = require("../../../config/database-connection");
const getCurrentDateTime = require("../../../config/datetime.js");
const {
  main_cash_gl_acc_id,
  main_bank_gl_acc_id,
  lab_test_gl_acc_id,
  lab_test_loan_gl_acc_id,
  doctor_ref_fee_gl_acc_id,
  hospital_charge_gl_acc_id,
} = require("../../../config/account-config");

// ! get all pl account by type query
const allPLAccountByType = (req, res) => {
  const type = req.params.type;

  // Assuming the column name for type in pl_acc_tbl is 'type_column_name'. Replace it with the actual column name if different.
  db_con.query(
    "SELECT pl_acc_no FROM pl_acc_tbl WHERE pl_acc_type=?",
    [type],
    (error, results) => {
      if (error) {
        console.error(error); // Log the error for debugging purposes
        res.status(500).send(error);
      } else {
        if (results.length === 0) {
          // No data found for the specified type
          res.status(204).json({ message: "No data found" });
        } else {
          res.status(200).json(results);
        }
      }
    }
  );
};

// ! get all lab test transaction type = y query
const allLabTestTransaction = (req, res) => {
  const type = "Y";

  // Assuming the column name for type in pl_acc_tbl is 'type_column_name'. Replace it with the actual column name if different.
  db_con.query(
    "SELECT lab_tra_id,lab_tra_cus_acc,lab_tra_cus_id FROM lab_test_tra_tbl WHERE lab_tra_status=? GROUP BY lab_tra_id",
    [type],
    (error, results) => {
      if (error) {
        console.error(error); // Log the error for debugging purposes
        res.status(500).send(error);
      } else {
        if (results.length === 0) {
          // No data found for the specified type
          res.status(204).json({ message: "No data found" });
        } else {
          res.status(200).json(results);
        }
      }
    }
  );
};

// ! get expenses and income type code in gl account
const getGLAccountMainTypes = (req, res) => {
  db_con.query(
    "SELECT gltp_mtyp_id,gltp_mintyp_eng FROM gl_acctyp_tbl WHERE gltp_mtyp_id = '2' OR gltp_mtyp_id = '3' GROUP BY gltp_mtyp_id",
    (error, results) => {
      if (error) {
        console.error(error); // Log the error for debugging purposes
        res.status(500).send(error);
      } else {
        if (results.length === 0) {
          // No data found for the specified main type ID
          res.status(204).json({ message: "No data found" });
        } else {
          // Send the retrieved GL account types as a JSON response
          res.status(200).json(results);
        }
      }
    }
  );
};

// ! get all gl account sub type by main type id
const getGLAccountTypesByMainTypeId = (req, res) => {
  const mainTypeId = req.params.mainTypeId;

  // Assuming `gltp_mtyp_id` is the correct column name in the table. Replace it with the actual column name if different.
  db_con.query(
    "SELECT gltp_id, gltp_typ_eng, gltp_mtyp_id FROM gl_acctyp_tbl WHERE gltp_mtyp_id = ? GROUP BY gltp_id",
    [mainTypeId],
    (error, results) => {
      if (error) {
        console.error(error); // Log the error for debugging purposes
        res.status(500).send(error);
      } else {
        if (results.length === 0) {
          // No data found for the specified main type ID
          res.status(204).json({ message: "No data found" });
        } else {
          // Send the retrieved GL account types as a JSON response
          res.status(200).json(results);
        }
      }
    }
  );
};

// ! get all gl account names and id by type
const getAllGLAccountByType = (req, res) => {
  const typeId = req.params.type;

  db_con.query(
    "SELECT glacc_id,glacc_name_eng FROM gl_acc_tbl INNER JOIN gl_acctyp_tbl ON (gl_acc_tbl.gl_typid = gl_acctyp_tbl.gltp_id) WHERE gltp_mtyp_id = ?",
    [typeId],
    (error, results) => {
      if (error) {
        console.error(error); // Log the error for debugging purposes
        res.status(500).send(error);
      } else {
        if (results.length === 0) {
          // No data found for the specified main type ID
          res.status(204).json({ message: "No data found" });
        } else {
          // Send the retrieved GL account types as a JSON response
          res.status(200).json(results);
        }
      }
    }
  );
};

// ? view all gl accounts in table view
const getAllGLAccountsToTableView = async (req, res) => {
  try {
    const { draw, start, length, search, order, columns } = req.query;
    const page = Math.ceil(start / length) + 1;
    const limit = +length;
    const offset = (page - 1) * limit;

    const orderColumnIndex = order[0].column;
    const orderDirection = order[0].dir;
    const orderColumn = columns[orderColumnIndex].data;

    const searchPattern = `%${search.value}%`;
    let searchWhere = "";
    const searchParams = [];

    if (search.value) {
      searchWhere = " WHERE ";
      // Add your search conditions here based on table columns
      searchWhere += `
        gltp_mintyp_eng LIKE ? OR
        gltp_subtyp_eng LIKE ? OR
        glacc_name_eng LIKE ? OR
        glacc_id LIKE ?
      `;
      searchParams.push(
        searchPattern,
        searchPattern,
        searchPattern,
        searchPattern
      );
    }

    const selectQuery = `
      SELECT
        gltp_mintyp_eng,
        gltp_subtyp_eng,
        glacc_name_eng,
        glacc_id
      FROM
        gl_acc_tbl
        INNER JOIN gl_acctyp_tbl
          ON (
            gl_acc_tbl.gl_typid = gl_acctyp_tbl.gltp_id
          )
      ${searchWhere}
      ORDER BY ${orderColumn} ${orderDirection}
      LIMIT ?, ?
    `;

    const countQuery = `
      SELECT COUNT(glacc_id) AS total 
      FROM gl_acc_tbl
      INNER JOIN gl_acctyp_tbl
        ON (
          gl_acc_tbl.gl_typid = gl_acctyp_tbl.gltp_id
        )
      ${searchWhere}
    `;

    db_con.query(
      selectQuery,
      [...searchParams, offset, limit],
      (error, results) => {
        if (error) {
          return res.status(500).send(error);
        }

        if (results.length === 0) {
          return res.status(200).json({
            draw: +draw,
            recordsTotal: 0,
            recordsFiltered: 0,
            data: [],
            message: "No data found in the table.",
          });
        }

        db_con.query(countQuery, searchParams, (countError, countResults) => {
          if (countError) {
            return res.status(500).send(countError);
          }

          const totalRecords = countResults[0].total;
          const response = {
            draw: +draw,
            recordsTotal: totalRecords,
            recordsFiltered: totalRecords,
            data: results,
          };

          res.status(200).json(response);
        });
      }
    );
  } catch (error) {
    res.status(500).send(error.message);
  }
};

// ? create new gl account
const createGLAccount = async (req, res) => {
  try {
    const { gl_typid, glacc_name_eng } = req.body;

    // ? Format the GL Type ID with leading zeros and prefix "GL"
    const formattedGLTypid = `GL${String(gl_typid).padStart(3, "0")}-`;

    // ? Check if the GL Account name already exists
    const queryCheckName =
      "SELECT COUNT(*) AS count FROM gl_acc_tbl WHERE glacc_name_eng = ?";

    db_con.query(
      queryCheckName,
      [glacc_name_eng],
      (checkError, checkResults) => {
        if (checkError) {
          return res.status(500).send(checkError);
        }

        const nameExists = checkResults[0].count > 0;

        if (nameExists) {
          return res
            .status(400)
            .json({ error: "GL Account name already exists" });
        }

        // ? Get the maximum ID from existing GL accounts
        const queryMaxId =
          "SELECT MAX(CAST(SUBSTRING(glacc_id, 7) AS UNSIGNED)) AS max_id FROM gl_acc_tbl WHERE gl_typid = ?";

        // ? Pass gl_typid as an array
        db_con.query(queryMaxId, [gl_typid], (maxIdError, maxIdResults) => {
          if (maxIdError) {
            return res.status(500).send(maxIdError);
          }

          const maxID = maxIdResults[0].max_id || 0;
          const nextID = maxID + 1;

          // ? Format the next GL account ID
          const acc_id = `${formattedGLTypid}${String(nextID).padStart(
            4,
            "0"
          )}`;

          // ? Insert into gl_acc_tbl
          const insertQuery =
            "INSERT INTO gl_acc_tbl (glacc_id, glacc_name_eng, gl_typid) VALUES (?, ?, ?)";

          const insertParams = [acc_id, glacc_name_eng, gl_typid];

          db_con.query(insertQuery, insertParams, (insertError) => {
            if (insertError) {
              return res.status(500).send(insertError);
            }

            console.log("GL Account created successfully");
            res.status(200).json({
              success: true,
              message: "GL Account created successfully",
              glacc_id: acc_id,
            });
          });
        });
      }
    );
  } catch (error) {
    res.status(500).send(error.message);
  }
};

// ? view all transaction by selected account
const getTransactionsToTableViewByID = async (req, res) => {
  const glTraCodeParam = req.params.id;
  try {
    const { draw, start, length, search, order, columns } = req.query;
    const page = Math.ceil(start / length) + 1;
    const limit = +length;
    const offset = (page - 1) * limit;

    const orderColumnIndex = order[0].column;
    const orderDirection = order[0].dir;
    const orderColumn = columns[orderColumnIndex].data;

    const searchPattern = `%${search.value}%`;
    let searchWhere = "";
    const searchParams = [];

    if (search.value) {
      searchWhere = " WHERE ";
      // Add your search conditions here based on table columns
      searchWhere += `
        gl_tra_id LIKE ? OR
        gl_tra_description LIKE ? OR
        user_fname LIKE ? OR
        brach_name LIKE ? OR
        gl_tra_date LIKE ?
      `;
      searchParams.push(
        searchPattern,
        searchPattern,
        searchPattern,
        searchPattern,
        searchPattern
      );
    }

    if (glTraCodeParam) {
      // Add a condition for gl_tra_code
      searchWhere += ` AND gl_tra_code = ? AND gl_tra_status = 'Y'`;
      searchParams.push(glTraCodeParam);
    }

    const selectQuery = `
      SELECT
        gl_tra_id,
        glacc_name_eng,
        gl_tra_debit,
        gl_tra_credit,
        CONCAT(date_format(gl_tra_date, '%Y-%m-%d'), ' ', date_format(gl_tra_time, '%H:%m:%s')) AS gl_tra_date,
        gl_tra_description,
        user_fname,
        brach_name
      FROM
        gl_tra_tbl
        INNER JOIN gl_acc_tbl
          ON (gl_tra_tbl.gl_tra_code = gl_acc_tbl.glacc_id)
        INNER JOIN sys_user_tbl
          ON (gl_tra_tbl.gl_tra_user = sys_user_tbl.user_id)
        INNER JOIN sys_branch_tbl
          ON (gl_tra_tbl.gl_tra_branch = sys_branch_tbl.brach_id)
      ${searchWhere}
      ORDER BY ${orderColumn} ${orderDirection}
      LIMIT ?, ?
    `;

    const countQuery = `
      SELECT COUNT(gl_tra_id) AS total 
      FROM gl_tra_tbl
      INNER JOIN gl_acc_tbl
        ON (gl_tra_tbl.gl_tra_code = gl_acc_tbl.glacc_id)
      INNER JOIN sys_user_tbl
        ON (gl_tra_tbl.gl_tra_user = sys_user_tbl.user_id)
      INNER JOIN sys_branch_tbl
        ON (gl_tra_tbl.gl_tra_branch = sys_branch_tbl.brach_id)
      ${searchWhere}
    `;

    db_con.query(
      selectQuery,
      [...searchParams, offset, limit],
      (error, results) => {
        if (error) {
          return res.status(500).send(error);
        }

        if (results.length === 0) {
          return res.status(200).json({
            draw: +draw,
            recordsTotal: 0,
            recordsFiltered: 0,
            data: [],
            message: "No data found in the table.",
          });
        }

        db_con.query(countQuery, searchParams, (countError, countResults) => {
          if (countError) {
            return res.status(500).send(countError);
          }

          const totalRecords = countResults[0].total;
          const response = {
            draw: +draw,
            recordsTotal: totalRecords,
            recordsFiltered: totalRecords,
            data: results,
          };

          res.status(200).json(response);
        });
      }
    );
  } catch (error) {
    res.status(500).send(error.message);
  }
};

// ? calculating GL balance credit debit sum summery by account id
const calculateGLBalanceByAccID = (req, res) => {
  const glTraCode = req.params.id;
  // Assuming `gl_tra_code` is the correct column name in the table. Replace it with the actual column name if different.
  db_con.query(
    "SELECT COALESCE(SUM(gl_tra_debit), 0) AS totalDebit, COALESCE(SUM(gl_tra_credit), 0) AS totalCredit, COALESCE(SUM(gl_tra_debit - gl_tra_credit), 0) AS expense_balance, COALESCE(SUM(gl_tra_credit - gl_tra_debit), 0) AS income_balance FROM gl_tra_tbl WHERE gl_tra_code = ? AND gl_tra_status='Y'",
    [glTraCode],
    (error, results) => {
      if (error) {
        console.error(error);
        res.status(500).send(error);
      } else {
        // Assuming you want to return the calculated balances as JSON
        res.status(200).json(results[0]);
      }
    }
  );
};

// ? get income expense and cash bank balance daily
const calculateGLIncomeExpenseCashBankByDailyDashboard = (req, res) => {
  const dateTime = getCurrentDateTime();

  // Construct the SQL query using template literals
  const sqlQuery = `
    SELECT
      COALESCE(SUM(CASE WHEN gl_tra_type = 'V' AND gl_tra_status = 'Y' THEN gl_tra_credit ELSE 0 END), 0) AS total_expense,
      COALESCE(SUM(CASE WHEN gl_tra_type = 'R' AND gl_tra_status = 'Y' THEN gl_tra_debit ELSE 0 END), 0) AS total_income,
      COALESCE(SUM(CASE WHEN gl_tra_code = '${main_cash_gl_acc_id}' AND gl_tra_status = 'Y' THEN (gl_tra_debit - gl_tra_credit) ELSE 0 END), 0) AS net_balance_cash,
      COALESCE(SUM(CASE WHEN gl_tra_code = '${main_bank_gl_acc_id}' AND gl_tra_status = 'Y' THEN (gl_tra_debit - gl_tra_credit) ELSE 0 END), 0) AS net_balance_bank
    FROM
      gl_tra_tbl
    WHERE
      DATE(gl_tra_date) = '${dateTime.date}';
  `;

  // Execute the SQL query
  db_con.query(sqlQuery, (error, results) => {
    if (error) {
      console.error(error);
      res.status(500).send(error); // Handle error response
    } else {
      res.status(200).json(results[0]); // Send JSON response with the calculated balances
    }
  });
};

module.exports = {
  allPLAccountByType,
  allLabTestTransaction,
  getGLAccountTypesByMainTypeId,
  getGLAccountMainTypes,
  getAllGLAccountsToTableView,
  createGLAccount,
  getAllGLAccountByType,
  getTransactionsToTableViewByID,
  calculateGLBalanceByAccID,
  calculateGLIncomeExpenseCashBankByDailyDashboard,
};
