const db_con = require("../../../config/database-connection");
const getCurrentDateTime = require("../../../config/datetime.js");
const {
  main_cash_gl_acc_id,
  main_bank_gl_acc_id,
  lab_test_gl_acc_id,
  lab_test_loan_gl_acc_id,
  doctor_ref_fee_gl_acc_id,
  hospital_charge_gl_acc_id,
} = require("../../../config/account-config");

// ? get income expense and cash bank balance monthly date rage
const calculateGLIncomeExpenseGrossProfitMonthlyDashboard = (req, res) => {
  // ? Extract start date, end date, and branchId from request parameters
  const { startDate, endDate, branchId } = req.params;

  // ? Validate that startDate, endDate, and branchId are provided
  if (!startDate || !endDate || !branchId) {
    res.status(400).json({
      error: "startDate, endDate, and branchId parameters are required.",
    });
    return;
  }

  // ? Construct the SQL query using template literals
  const sqlQuery = `
    SELECT
        COALESCE(SUM(CASE WHEN gl_tra_type = 'V' AND gl_tra_status = 'Y' THEN gl_tra_credit ELSE 0 END), 0) AS total_expense,
        COALESCE(SUM(CASE WHEN gl_tra_type = 'R' AND gl_tra_status = 'Y' THEN gl_tra_debit ELSE 0 END), 0) AS total_income,
        (COALESCE(SUM(CASE WHEN gl_tra_type = 'R' AND gl_tra_status = 'Y' THEN gl_tra_debit ELSE 0 END), 0) -
         COALESCE(SUM(CASE WHEN gl_tra_type = 'V' AND gl_tra_status = 'Y' THEN gl_tra_credit ELSE 0 END), 0)) AS gross_profit
    FROM
        gl_tra_tbl
    WHERE
        DATE(gl_tra_date) >= '${startDate}' AND DATE(gl_tra_date) <= '${endDate}' AND
        (gl_tra_branch = '${branchId}' OR gl_tra_branch LIKE '${branchId}%');
`;

  // ? Execute the SQL query
  db_con.query(sqlQuery, (error, results) => {
    if (error) {
      console.error(error);
      res.status(500).send(error); // Handle error response
    } else {
      res.status(200).json(results[0]); // Send JSON response with the calculated balances
    }
  });
};

// ? get income expense and cash bank balance daily
const calculateGLIncomeExpenseCashBankByMonthlyDashboard = (req, res) => {
  // ? Extract start date, end date, and branchId from request parameters
  const { startDate, endDate, branchId } = req.params;

  // ? Validate that startDate, endDate, and branchId are provided
  if (!startDate || !endDate || !branchId) {
    res.status(400).json({
      error: "startDate, endDate, and branchId parameters are required.",
    });
    return;
  }
  // Construct the SQL query using template literals
  const sqlQuery = `
    SELECT
      COALESCE(SUM(CASE WHEN gl_tra_type = 'V' AND gl_tra_status = 'Y' THEN gl_tra_credit ELSE 0 END), 0) AS total_expense,
      COALESCE(SUM(CASE WHEN gl_tra_type = 'R' AND gl_tra_status = 'Y' THEN gl_tra_debit ELSE 0 END), 0) AS total_income,
      COALESCE(SUM(CASE WHEN gl_tra_code = '${main_cash_gl_acc_id}' AND gl_tra_status = 'Y' THEN (gl_tra_debit - gl_tra_credit) ELSE 0 END), 0) AS net_balance_cash,
      COALESCE(SUM(CASE WHEN gl_tra_code = '${main_bank_gl_acc_id}' AND gl_tra_status = 'Y' THEN (gl_tra_debit - gl_tra_credit) ELSE 0 END), 0) AS net_balance_bank,
      COALESCE(SUM(CASE WHEN gl_tra_code = '${main_cash_gl_acc_id}' AND gl_tra_type = 'V' AND gl_tra_status = 'Y' THEN gl_tra_credit ELSE 0 END), 0) AS net_expense_balance_cash,
      COALESCE(SUM(CASE WHEN gl_tra_code = '${main_bank_gl_acc_id}' AND gl_tra_type = 'V' AND gl_tra_status = 'Y' THEN gl_tra_credit ELSE 0 END), 0) AS net_expense_balance_bank,
      COALESCE(SUM(CASE WHEN gl_tra_code = '${main_cash_gl_acc_id}' AND gl_tra_type = 'R' AND gl_tra_status = 'Y' THEN gl_tra_debit ELSE 0 END), 0) AS net_income_balance_cash,
      COALESCE(SUM(CASE WHEN gl_tra_code = '${main_bank_gl_acc_id}' AND gl_tra_type = 'R' AND gl_tra_status = 'Y' THEN gl_tra_debit ELSE 0 END), 0) AS net_income_balance_bank
    FROM
      gl_tra_tbl
    WHERE
      DATE(gl_tra_date) >= '${startDate}' AND DATE(gl_tra_date) <= '${endDate}' AND
      (gl_tra_branch = '${branchId}' OR gl_tra_branch LIKE '${branchId}%');
  `;

  // Execute the SQL query
  db_con.query(sqlQuery, (error, results) => {
    if (error) {
      console.error(error);
      res.status(500).send(error); // Handle error response
    } else {
      res.status(200).json(results[0]); // Send JSON response with the calculated balances
    }
  });
};

// ? get income expense and cash bank ratio monthly date rage
const calculateGLIncomeExpenseCashVsBankRatioMonthlyDashboard = (req, res) => {
  // ? Extract start date, end date, and branchId from request parameters
  const { startDate, endDate, branchId } = req.params;

  // ? Validate that startDate, endDate, and branchId are provided
  if (!startDate || !endDate || !branchId) {
    res.status(400).json({
      error: "startDate, endDate, and branchId parameters are required.",
    });
    return;
  }

  // ? Construct the SQL query using template literals
  const sqlQuery = `
    SELECT
      COALESCE(SUM(CASE WHEN gl_tra_code = '${main_cash_gl_acc_id}' AND gl_tra_type = 'R' AND gl_tra_status = 'Y' THEN gl_tra_debit ELSE 0 END), 0) AS balance_cash_income,
      COALESCE(SUM(CASE WHEN gl_tra_code = '${main_bank_gl_acc_id}' AND gl_tra_type = 'R' AND gl_tra_status = 'Y' THEN gl_tra_debit ELSE 0 END), 0) AS balance_bank_income,
      COALESCE(SUM(CASE WHEN gl_tra_code = '${main_cash_gl_acc_id}' AND gl_tra_type = 'V' AND gl_tra_status = 'Y' THEN gl_tra_credit ELSE 0 END), 0) AS balance_cash_expenses,
      COALESCE(SUM(CASE WHEN gl_tra_code = '${main_bank_gl_acc_id}' AND gl_tra_type = 'V' AND gl_tra_status = 'Y' THEN gl_tra_credit ELSE 0 END), 0) AS balance_bank_expenses,
      COALESCE(SUM(CASE WHEN gl_tra_type = 'V' AND gl_tra_status = 'Y' THEN gl_tra_credit ELSE 0 END), 0) AS total_expense,
      COALESCE(SUM(CASE WHEN gl_tra_type = 'R' AND gl_tra_status = 'Y' THEN gl_tra_debit ELSE 0 END), 0) AS total_income
  FROM
      gl_tra_tbl
  WHERE
      DATE(gl_tra_date) >= '${startDate}' AND DATE(gl_tra_date) <= '${endDate}' AND
      (gl_tra_branch = '${branchId}' OR gl_tra_branch LIKE '${branchId}%');
  `;

  // ? Execute the SQL query
  db_con.query(sqlQuery, (error, results) => {
    if (error) {
      console.error(error);
      res.status(500).send(error); // Handle error response
    } else {
      res.status(200).json(results[0]); // Send JSON response with the calculated balances
    }
  });
};

// ? get income ration by accounts
const calculateGLIncomeAccountRatioMonthlyDashboard = (req, res) => {
  // Extract start date, end date, and branchId from request parameters
  const { startDate, endDate, branchId } = req.params;

  // Validate that startDate, endDate, and branchId are provided
  if (!startDate || !endDate || !branchId) {
    return res.status(400).json({
      error: "startDate, endDate, and branchId parameters are required.",
    });
  }

  // Validate that startDate is before endDate
  if (new Date(startDate) > new Date(endDate)) {
    return res.status(400).json({
      error: "startDate must be before endDate.",
    });
  }

  // Construct the parameterized SQL query
  const sqlQuery = `
  SELECT
      COALESCE(SUM(CASE WHEN gl_tra_type = 'R' AND gl_tra_status = 'Y' THEN gl_tra_credit ELSE 0 END), 0) AS total_income,
      glacc_name_eng AS account_name
  FROM
      gl_tra_tbl
  INNER JOIN gl_acc_tbl ON gl_tra_tbl.gl_tra_code = gl_acc_tbl.glacc_id
  WHERE
      DATE(gl_tra_date) BETWEEN ? AND ? AND
      (gl_tra_branch = ? OR gl_tra_branch LIKE ?)
  GROUP BY gl_tra_code
      HAVING total_income > 0;
  `;

  // Execute the parameterized SQL query
  db_con.query(
    sqlQuery,
    [startDate, endDate, branchId, `${branchId}%`],
    (error, results) => {
      if (error) {
        console.error(error);
        return res.status(500).send(error); // Handle error response
      } else {
        return res.status(200).json(results); // Send JSON response with all calculated balances
      }
    }
  );
};

// ? get expenses ration by accounts
const calculateGLExpensesAccountRatioMonthlyDashboard = (req, res) => {
  // Extract start date, end date, and branchId from request parameters
  const { startDate, endDate, branchId } = req.params;

  // Validate that startDate, endDate, and branchId are provided
  if (!startDate || !endDate || !branchId) {
    return res.status(400).json({
      error: "startDate, endDate, and branchId parameters are required.",
    });
  }

  // Validate that startDate is before endDate
  if (new Date(startDate) > new Date(endDate)) {
    return res.status(400).json({
      error: "startDate must be before endDate.",
    });
  }

  // Construct the parameterized SQL query
  const sqlQuery = `
  SELECT
      COALESCE(SUM(CASE WHEN gl_tra_type = 'V' AND gl_tra_status = 'Y' THEN gl_tra_debit ELSE 0 END), 0) AS total_income,
      glacc_name_eng AS account_name
  FROM
      gl_tra_tbl
  INNER JOIN gl_acc_tbl ON gl_tra_tbl.gl_tra_code = gl_acc_tbl.glacc_id
  WHERE
      DATE(gl_tra_date) BETWEEN ? AND ? AND
      (gl_tra_branch = ? OR gl_tra_branch LIKE ?)
  GROUP BY gl_tra_code
      HAVING total_income > 0;
  `;

  // Execute the parameterized SQL query
  db_con.query(
    sqlQuery,
    [startDate, endDate, branchId, `${branchId}%`],
    (error, results) => {
      if (error) {
        console.error(error);
        return res.status(500).send(error); // Handle error response
      } else {
        return res.status(200).json(results); // Send JSON response with all calculated balances
      }
    }
  );
};

// ? get all gl transaction by type
const getAllGLTraTransactionByType = async (req, res) => {
  try {
    const { draw, start, length, search, order, columns } = req.query;
    const { startDate, endDate, branchId, type } = req.params;
    const page = Math.ceil(start / length) + 1;
    const limit = +length;
    const offset = (page - 1) * limit;

    const orderColumnIndex = order[0].column;
    const orderDirection = order[0].dir;
    const orderColumn = columns[orderColumnIndex].data;

    const searchPattern = `%${search.value}%`;
    let searchWhere = "";
    const searchParams = [];

    if (search.value) {
      searchWhere = " WHERE ";
      searchWhere += " (gl_tra_id LIKE ? ";
      searchWhere += " OR gl_tra_debit LIKE ? ";
      searchWhere += " OR gl_tra_credit LIKE ? ";
      searchWhere += " OR gl_tra_date LIKE ? ";
      searchWhere += " OR user_fname LIKE ? ";
      searchWhere += " OR brach_name LIKE ? ";
      searchWhere += " OR gl_tra_description LIKE ? ) ";
      searchParams.push(
        searchPattern,
        searchPattern,
        searchPattern,
        searchPattern,
        searchPattern,
        searchPattern,
        searchPattern
      );
    }

    if (type) {
      searchWhere += searchWhere ? " AND " : " WHERE ";
      searchWhere += `DATE(gl_tra_date) >= '${startDate}' AND DATE(gl_tra_date) <= '${endDate}' AND
      (gl_tra_branch = '${branchId}' OR gl_tra_branch LIKE '${branchId}%') AND (gl_tra_code = '${main_cash_gl_acc_id}' OR gl_tra_code = '${main_bank_gl_acc_id}') AND gl_tra_status = 'Y' AND gl_tra_type = ?`;
      searchParams.push(type);
    }

    const selectQuery = `
    SELECT
        gl_tra_id,
        gl_tra_code,
        glacc_name_eng,
        gl_tra_debit,
        gl_tra_credit,
        gl_tra_type,
        DATE_FORMAT(gl_tra_date, '%Y-%m-%d') AS gl_tra_date,
        DATE_FORMAT(gl_tra_time, '%H:%i:%s') AS gl_tra_time,
        gl_tra_description,
        gl_tra_branch,
        gl_tra_user,
        user_fname,
        brach_name
        FROM
        gl_tra_tbl
    INNER JOIN gl_acc_tbl
        ON gl_tra_tbl.gl_tra_code = gl_acc_tbl.glacc_id
    INNER JOIN sys_user_tbl
        ON gl_tra_tbl.gl_tra_user = sys_user_tbl.user_id
    INNER JOIN sys_branch_tbl
        ON gl_tra_tbl.gl_tra_branch = sys_branch_tbl.brach_id
    ${searchWhere}
    ORDER BY ${orderColumn} ${orderDirection}
    LIMIT ?, ?
    `;

    const countQuery = `
    SELECT COUNT(gl_tra_id) AS total 
    FROM
    gl_tra_tbl
    INNER JOIN gl_acc_tbl
        ON gl_tra_tbl.gl_tra_code = gl_acc_tbl.glacc_id
    INNER JOIN sys_user_tbl
        ON gl_tra_tbl.gl_tra_user = sys_user_tbl.user_id
    INNER JOIN sys_branch_tbl
        ON gl_tra_tbl.gl_tra_branch = sys_branch_tbl.brach_id
    ${searchWhere};
      `;

    db_con.query(
      selectQuery,
      [...searchParams, offset, limit],
      (error, results) => {
        if (error) {
          return res.status(500).send(error);
        }

        if (results.length === 0) {
          return res.status(200).json({
            draw: +draw,
            recordsTotal: 0,
            recordsFiltered: 0,
            data: [],
            message: "No data found in the table.",
          });
        }

        db_con.query(countQuery, searchParams, (countError, countResults) => {
          if (countError) {
            return res.status(500).send(countError);
          }

          const totalRecords = countResults[0].total;
          const response = {
            draw: +draw,
            recordsTotal: totalRecords,
            recordsFiltered: totalRecords,
            data: results,
          };

          res.status(200).json(response);
        });
      }
    );
  } catch (error) {
    res.status(500).send(error.message);
  }
};

// ? view all gl income and expenses transaction
const getAllGLTraTransactionByExpensesIncomeType = async (req, res) => {
  try {
    const { draw, start, length, search, order, columns } = req.query;
    const { startDate, endDate, branchId, type } = req.params;
    const page = Math.ceil(start / length) + 1;
    const limit = +length;
    const offset = (page - 1) * limit;

    const orderColumnIndex = order[0].column;
    const orderDirection = order[0].dir;
    const orderColumn = columns[orderColumnIndex].data;

    const searchPattern = `%${search.value}%`;
    let searchWhere = "";
    const searchParams = [];

    if (search.value) {
      searchWhere = " WHERE ";
      searchWhere += " (gl_tra_id LIKE ? ";
      searchWhere += " OR gl_tra_debit LIKE ? ";
      searchWhere += " OR gl_tra_credit LIKE ? ";
      searchWhere += " OR gl_tra_date LIKE ? ";
      searchWhere += " OR user_fname LIKE ? ";
      searchWhere += " OR brach_name LIKE ? ";
      searchWhere += " OR gl_tra_description LIKE ? ) ";
      searchParams.push(
        searchPattern,
        searchPattern,
        searchPattern,
        searchPattern,
        searchPattern,
        searchPattern,
        searchPattern
      );
    }

    searchWhere += searchWhere ? " AND " : " WHERE ";
    searchWhere += `DATE(gl_tra_date) >= '${startDate}' AND DATE(gl_tra_date) <= '${endDate}' AND
      (gl_tra_branch = '${branchId}' OR gl_tra_branch LIKE '${branchId}%') AND 
      (gl_tra_code = '${main_cash_gl_acc_id}' OR gl_tra_code = '${main_bank_gl_acc_id}') AND gl_tra_status = 'Y' AND
      (gl_tra_type = 'R' OR gl_tra_type = 'V')`;

    const selectQuery = `
    SELECT
        gl_tra_id,
        gl_tra_code,
        glacc_name_eng,
        gl_tra_debit,
        gl_tra_credit,
        gl_tra_type,
        DATE_FORMAT(gl_tra_date, '%Y-%m-%d') AS gl_tra_date,
        DATE_FORMAT(gl_tra_time, '%H:%i:%s') AS gl_tra_time,
        gl_tra_description,
        gl_tra_branch,
        gl_tra_user,
        user_fname,
        brach_name
        FROM
        gl_tra_tbl
    INNER JOIN gl_acc_tbl
        ON gl_tra_tbl.gl_tra_code = gl_acc_tbl.glacc_id
    INNER JOIN sys_user_tbl
        ON gl_tra_tbl.gl_tra_user = sys_user_tbl.user_id
    INNER JOIN sys_branch_tbl
        ON gl_tra_tbl.gl_tra_branch = sys_branch_tbl.brach_id
    ${searchWhere}
    ORDER BY ${orderColumn} ${orderDirection}
    LIMIT ?, ?
    `;

    const countQuery = `
    SELECT COUNT(gl_tra_id) AS total 
    FROM
    gl_tra_tbl
    INNER JOIN gl_acc_tbl
        ON gl_tra_tbl.gl_tra_code = gl_acc_tbl.glacc_id
    INNER JOIN sys_user_tbl
        ON gl_tra_tbl.gl_tra_user = sys_user_tbl.user_id
    INNER JOIN sys_branch_tbl
        ON gl_tra_tbl.gl_tra_branch = sys_branch_tbl.brach_id
    ${searchWhere};
      `;

    db_con.query(
      selectQuery,
      [...searchParams, offset, limit],
      (error, results) => {
        if (error) {
          return res.status(500).send(error);
        }

        if (results.length === 0) {
          return res.status(200).json({
            draw: +draw,
            recordsTotal: 0,
            recordsFiltered: 0,
            data: [],
            message: "No data found in the table.",
          });
        }

        db_con.query(countQuery, searchParams, (countError, countResults) => {
          if (countError) {
            return res.status(500).send(countError);
          }

          const totalRecords = countResults[0].total;
          const response = {
            draw: +draw,
            recordsTotal: totalRecords,
            recordsFiltered: totalRecords,
            data: results,
          };

          res.status(200).json(response);
        });
      }
    );
  } catch (error) {
    res.status(500).send(error.message);
  }
};

// ? view all gl income and summery
const getAllGLTraTransactionSummery = async (req, res) => {
  try {
    const { draw, start, length, search, order, columns } = req.query;
    const { startDate, endDate, branchId, type } = req.params;
    const page = Math.ceil(start / length) + 1;
    const limit = +length;
    const offset = (page - 1) * limit;

    const orderColumnIndex = order[0].column;
    const orderDirection = order[0].dir;
    const orderColumn = columns[orderColumnIndex].data;

    const searchPattern = `%${search.value}%`;
    let searchWhere = "";
    const searchParams = [];

    if (search.value) {
      searchWhere = " WHERE ";
      searchWhere += " (glacc_name_eng LIKE ? ) ";
      searchParams.push(searchPattern);
    }

    searchWhere += searchWhere ? " AND " : " WHERE ";
    searchWhere += `DATE(gl_tra_date) >= '${startDate}' AND DATE(gl_tra_date) <= '${endDate}' AND
      (gl_tra_branch = '${branchId}' OR gl_tra_branch LIKE '${branchId}%') AND 
      gl_tra_status = 'Y'`;

    const selectQuery = `
    SELECT
        gl_tra_code,
        glacc_name_eng,
        SUM(gl_tra_debit) AS ttl_debit,
        SUM(gl_tra_credit) AS ttl_credit
        FROM
        gl_tra_tbl
    INNER JOIN gl_acc_tbl
        ON gl_tra_tbl.gl_tra_code = gl_acc_tbl.glacc_id
    INNER JOIN sys_user_tbl
        ON gl_tra_tbl.gl_tra_user = sys_user_tbl.user_id
    INNER JOIN sys_branch_tbl
        ON gl_tra_tbl.gl_tra_branch = sys_branch_tbl.brach_id
    ${searchWhere}
    GROUP BY gl_tra_code ORDER BY ${orderColumn} ${orderDirection}
    LIMIT ?, ?
    `;

    const countQuery = `
    SELECT COUNT(gl_tra_code) AS total 
    FROM
    gl_tra_tbl
    INNER JOIN gl_acc_tbl
        ON gl_tra_tbl.gl_tra_code = gl_acc_tbl.glacc_id
    INNER JOIN sys_user_tbl
        ON gl_tra_tbl.gl_tra_user = sys_user_tbl.user_id
    INNER JOIN sys_branch_tbl
        ON gl_tra_tbl.gl_tra_branch = sys_branch_tbl.brach_id
    ${searchWhere} GROUP BY gl_tra_code;
      `;

    db_con.query(
      selectQuery,
      [...searchParams, offset, limit],
      (error, results) => {
        if (error) {
          return res.status(500).send(error);
        }

        if (results.length === 0) {
          return res.status(200).json({
            draw: +draw,
            recordsTotal: 0,
            recordsFiltered: 0,
            data: [],
            message: "No data found in the table.",
          });
        }

        db_con.query(countQuery, searchParams, (countError, countResults) => {
          if (countError) {
            return res.status(500).send(countError);
          }

          const totalRecords = countResults[0].total;
          const response = {
            draw: +draw,
            recordsTotal: totalRecords,
            recordsFiltered: totalRecords,
            data: results,
          };

          res.status(200).json(response);
        });
      }
    );
  } catch (error) {
    res.status(500).send(error.message);
  }
};

module.exports = {
  calculateGLIncomeExpenseGrossProfitMonthlyDashboard,
  calculateGLIncomeExpenseCashVsBankRatioMonthlyDashboard,
  calculateGLIncomeAccountRatioMonthlyDashboard,
  calculateGLExpensesAccountRatioMonthlyDashboard,
  getAllGLTraTransactionByType,
  getAllGLTraTransactionByExpensesIncomeType,
  getAllGLTraTransactionSummery,
  calculateGLIncomeExpenseCashBankByMonthlyDashboard,
};
