const db_con = require("../../../config/database-connection");
const getCurrentDateTime = require("../../../config/datetime.js");
const {
  main_cash_gl_acc_id,
  main_bank_gl_acc_id,
  lab_test_gl_acc_id,
  lab_test_loan_gl_acc_id,
  doctor_ref_fee_gl_acc_id,
  hospital_charge_gl_acc_id,
} = require("../../../config/account-config");

// ! get all lab test transaction data for table view
const getAllLabTestByHospitalLabTestPatientBranchStartEndDateDataTable = async (
  req,
  res
) => {
  try {
    const { draw, start, length, search, order, columns } = req.query;
    const { branchId, labTestId, hospitalId, patId, startDate, endDate } =
      req.params; // Extracting traID from parameters
    const page = Math.ceil(start / length) + 1;
    const limit = +length;
    const offset = (page - 1) * limit;

    const orderColumnIndex = order[0].column;
    const orderDirection = order[0].dir;
    const orderColumn = columns[orderColumnIndex].data;

    const searchPattern = `%${search.value}%`;
    let searchWhere = "";
    const searchParams = [];

    if (search.value) {
      searchWhere = " WHERE ";
      searchWhere += " (id LIKE ? ";
      searchWhere += " OR lab_tra_id LIKE ? ";
      searchWhere += " OR lab_tra_doct_id LIKE ? ";
      searchWhere += " OR lab_test_name LIKE ? ";
      searchWhere += " OR pat_name LIKE ? ";
      searchWhere += " OR hos_name LIKE ? ";
      searchWhere += " OR brach_name LIKE ?) ";
      searchParams.push(
        searchPattern,
        searchPattern,
        searchPattern,
        searchPattern,
        searchPattern,
        searchPattern,
        searchPattern
      );
    }

    // Adding filter for specific lab_tra_id value (B01-LT00000001)
    if (patId) {
      searchWhere += searchWhere ? " AND " : " WHERE ";
      searchWhere += `DATE(lab_tra_date) >= '${startDate}' AND DATE(lab_tra_date) <= '${endDate}' AND
      (lab_tra_branch_id = '${branchId}' OR lab_tra_branch_id LIKE '${branchId}%') AND
      (lab_tra_lab_test_id = '${labTestId}' OR lab_tra_lab_test_id LIKE '%${labTestId}%') AND
      (lab_tra_hospital_id = '${hospitalId}' OR lab_tra_hospital_id LIKE '${hospitalId}%') AND
      (lab_tra_cus_id = '${patId}' OR lab_tra_cus_id LIKE '${patId}%')`;
    }

    const selectQuery = `
        SELECT id, 
        lab_tra_id, 
        lab_tra_collect_datetime, 
        DATE_FORMAT(lab_tra_date, '%Y-%m-%d') AS lab_tra_date, 
        lab_tra_time, 
        lab_tra_cus_id, 
        lab_tra_cus_acc, 
        lab_tra_doct_id, 
        lab_tra_doct_acc, 
        lab_tra_doct_rate, 
        lab_tra_lab_test_id, 
        lab_tra_price, 
        lab_tra_actual_price, 
        lab_tra_discount_rate, 
        lab_tra_total_discount, 
        lab_tra_total_amount, 
        lab_tra_total_amount - lab_tra_total_discount AS amount,
        lab_tra_ref_fee, 
        lab_tra_ref_fee_update, 
        lab_tra_hospital_id, 
        lab_tra_hospital_fee, 
        lab_tra_hospital_fee_update, 
        lab_tra_data_cal_status, 
        lab_tra_lab_test_send, 
        DATE_FORMAT(lab_tra_lab_test_send_time, '%Y-%m-%d %H:%i:%s') AS lab_tra_lab_test_send_time, 
        lab_tra_lab_test_receive, 
        DATE_FORMAT(lab_tra_lab_test_receive_time, '%Y-%m-%d %H:%i:%s') AS lab_tra_lab_test_receive_time, 
        lab_tra_status, 
        lab_tra_branch_id, 
        lab_tra_user_id,
        lab_test_name,
        doct_name,
        hos_name,
        pat_name,
        brach_name,
        lab_tra_complete_status
        FROM lab_test_tra_tbl JOIN sys_lab_test_tbl ON lab_test_tra_tbl.lab_tra_lab_test_id = sys_lab_test_tbl.lab_id
        JOIN sys_doctor_tbl ON lab_test_tra_tbl.lab_tra_doct_id= sys_doctor_tbl.doct_id
        JOIN sys_hospital_tbl ON lab_test_tra_tbl.lab_tra_hospital_id = sys_hospital_tbl.hos_id
        JOIN sys_patient_tbl ON lab_test_tra_tbl.lab_tra_cus_id = sys_patient_tbl.pat_id
        JOIN sys_branch_tbl ON lab_test_tra_tbl.lab_tra_branch_id = sys_branch_tbl.brach_id
        ${searchWhere}
        ORDER BY ${orderColumn} ${orderDirection}
        LIMIT ?, ?
      `;

    const countQuery = `
        SELECT COUNT(id) AS total 
        FROM lab_test_tra_tbl JOIN sys_lab_test_tbl ON lab_test_tra_tbl.lab_tra_lab_test_id = sys_lab_test_tbl.lab_id
        JOIN sys_doctor_tbl ON lab_test_tra_tbl.lab_tra_doct_id= sys_doctor_tbl.doct_id
        JOIN sys_hospital_tbl ON lab_test_tra_tbl.lab_tra_hospital_id = sys_hospital_tbl.hos_id
        JOIN sys_patient_tbl ON lab_test_tra_tbl.lab_tra_cus_id = sys_patient_tbl.pat_id
        JOIN sys_branch_tbl ON lab_test_tra_tbl.lab_tra_branch_id = sys_branch_tbl.brach_id
        ${searchWhere}
      `;

    db_con.query(
      selectQuery,
      [...searchParams, offset, limit],
      (error, results) => {
        if (error) {
          return res.status(500).send(error);
        }

        if (results.length === 0) {
          return res.status(200).json({
            draw: +draw,
            recordsTotal: 0,
            recordsFiltered: 0,
            data: [],
            message: "No data found in the table.",
          });
        }

        db_con.query(countQuery, searchParams, (countError, countResults) => {
          if (countError) {
            return res.status(500).send(countError);
          }

          const totalRecords = countResults[0].total;
          const response = {
            draw: +draw,
            recordsTotal: totalRecords,
            recordsFiltered: totalRecords,
            data: results,
          };

          res.status(200).json(response);
        });
      }
    );
  } catch (error) {
    res.status(500).send(error.message);
  }
};

module.exports = {
  getAllLabTestByHospitalLabTestPatientBranchStartEndDateDataTable,
};
