const db_con = require("../../../config/database-connection");

// ! Get the last date for a given branch ID
const getLastDateByBranchId = (req, res) => {
  const branchId = req.params.id;

  const query = `
      SELECT DATE_FORMAT(sys_date, '%Y-%m-%d') AS formatted_date 
      FROM sys_day_end_tbl 
      WHERE sys_branch_id = ?
      ORDER BY sys_date DESC
      LIMIT 1
    `;

  db_con.query(query, [branchId], (error, results) => {
    if (error) {
      return res.status(500).send(error);
    }

    if (results.length === 0) {
      return res
        .status(404)
        .json({ message: "No date found for the specified branch ID" });
    }

    res.status(200).json({ sys_date: results[0].formatted_date });
  });
};

// ! Get the last date for a given branch ID
const createDayOpen = (req, res) => {
  const { openDate, branchID, currentDateTime } = req.body;

  // Fetch the maximum sys_date and its close status for the given branch
  db_con.query(
    "SELECT DATE_FORMAT(sys_date, '%Y-%m-%d') AS max_date,sys_day_close_status FROM sys_day_end_tbl WHERE sys_branch_id = ? ORDER BY sys_date DESC LIMIT 1",
    [branchID],
    (error, rows) => {
      if (error) {
        console.error("Error:", error);
        return res.status(500).json({ message: "Internal Server Error." });
      }

      const { max_date: maxDate, sys_day_close_status: closeStatus } =
        rows[0] || {};

      // If no previous entry exists for the branch, insert a new one
      if (!maxDate) {
        db_con.query(
          "INSERT INTO sys_day_end_tbl (sys_date, sys_branch_id, sys_day_open_time) VALUES (?, ?, ?)",
          [openDate, branchID, currentDateTime],
          (error) => {
            if (error) {
              console.error("Error:", error);
              return res
                .status(500)
                .json({ message: "Internal Server Error." });
            }
            return res
              .status(201)
              .json({ message: "Day initialized successfully." });
          }
        );
      }
      // If the previous entry exists, determine the action based on dates and statuses
      else {
        // If max date is later than the open date
        if (maxDate > openDate) {
          return res.status(400).json({
            message:
              "Cannot set the system date earlier than the last closed date.",
          });
        }
        // If max date matches the open date
        else if (maxDate === openDate) {
          // If the system is open on that date
          if (closeStatus === "N") {
            return res.status(200).json({ message: "Day open successfully." });
          }
          // If the system is closed on that date
          else {
            return res.status(200).json({ message: "Please login tomorrow." });
          }
        }
        // If max date is earlier than the open date
        else {
          // If the system is open on the max date
          if (closeStatus === "N") {
            return res
              .status(400)
              .json({ message: "Cannot open the system on this date." });
          }
          // If the system is closed on the max date
          else {
            db_con.query(
              'INSERT INTO sys_day_end_tbl (sys_date, sys_branch_id, sys_day_close_status) VALUES (?, ?, "N")',
              [openDate, branchID],
              (error) => {
                if (error) {
                  console.error("Error:", error);
                  return res
                    .status(500)
                    .json({ message: "Internal Server Error." });
                }
                return res
                  .status(201)
                  .json({ message: "Day initialized successfully." });
              }
            );
          }
        }
      }
    }
  );
};

module.exports = {
  getLastDateByBranchId,
  createDayOpen,
};
