const db_con = require("../../../config/database-connection");
const generateTestTraGenId = require("../../../Function/lab-test-transaction-function");
const getCurrentDateTime = require("../../../config/datetime.js");
const fs = require("fs");
const multer = require("multer");
const PDFDocument = require("pdfkit");

const insertLabTestTransaction = (req, res) => {
  let tra_id = "";
  const {
    lab_tra_id_tmp,
    lab_tra_collect_datetime_tmp,
    lab_tra_date_tmp,
    lab_tra_time_tmp,
    lab_tra_cus_id_tmp,
    lab_tra_cus_acc_tmp,
    lab_tra_doct_id_tmp,
    lab_tra_doct_acc_tmp,
    lab_tra_doct_rate_tmp,
    lab_tra_lab_test_id_tmp,
    lab_tra_price_tmp,
    lab_tra_actual_price_tmp,
    lab_tra_discount_rate_tmp,
    lab_tra_total_discount_tmp,
    lab_tra_total_amount_tmp,
    lab_tra_hospital_id_tmp,
    lab_tra_lab_test_send_time_tmp,
    lab_tra_lab_test_receive_time_tmp,
    lab_tra_branch_id_tmp,
    lab_tra_user_id_tmp,
  } = req.body;

  let query;
  let values;

  if (lab_tra_id_tmp === "") {
    generateTestTraGenId(
      db_con,
      lab_tra_branch_id_tmp,
      "LT",
      (error, newId) => {
        if (error) {
          console.error("Error:", error);
          return res
            .status(500)
            .json({ error: "Failed to generate transaction ID" });
        }
        tra_id = newId;
        setupAndExecuteQuery();
      }
    );
  } else {
    tra_id = lab_tra_id_tmp;
    setupAndExecuteQuery();
  }

  function setupAndExecuteQuery() {
    query = `
        INSERT INTO lab_test_tra_temp_tbl 
        (lab_tra_id_tmp, lab_tra_collect_datetime_tmp, lab_tra_date_tmp, lab_tra_time_tmp, lab_tra_cus_id_tmp, lab_tra_cus_acc_tmp, lab_tra_doct_id_tmp, lab_tra_doct_acc_tmp, lab_tra_doct_rate_tmp, lab_tra_lab_test_id_tmp, lab_tra_price_tmp, lab_tra_actual_price_tmp, lab_tra_discount_rate_tmp, lab_tra_total_discount_tmp, lab_tra_total_amount_tmp, lab_tra_hospital_id_tmp, lab_tra_lab_test_send_time_tmp, lab_tra_lab_test_receive_time_tmp, lab_tra_branch_id_tmp,lab_tra_user_id_tmp)
        VALUES (?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?)`;

    values = [
      tra_id,
      lab_tra_collect_datetime_tmp,
      lab_tra_date_tmp,
      lab_tra_time_tmp,
      lab_tra_cus_id_tmp,
      lab_tra_cus_acc_tmp,
      lab_tra_doct_id_tmp,
      lab_tra_doct_acc_tmp,
      lab_tra_doct_rate_tmp,
      lab_tra_lab_test_id_tmp,
      lab_tra_price_tmp,
      lab_tra_actual_price_tmp,
      lab_tra_discount_rate_tmp,
      lab_tra_total_discount_tmp,
      lab_tra_total_amount_tmp,
      lab_tra_hospital_id_tmp,
      lab_tra_lab_test_send_time_tmp,
      lab_tra_lab_test_receive_time_tmp,
      lab_tra_branch_id_tmp,
      lab_tra_user_id_tmp,
    ];

    db_con.query(query, values, (err, results) => {
      if (err) {
        console.error(err);
        return res
          .status(500)
          .json({ error: "Error inserting data", tra_id: tra_id });
      }
      res
        .status(201)
        .json({ message: "Data inserted successfully", tra_id: tra_id });
    });
  }
};

// ! get all not update test transactions query
const getAllNotUpdateTransactions = (req, res) => {
  // ? not update data get query
  db_con.query(
    "SELECT lab_tra_id_tmp,CONCAT(lab_tra_id_tmp, ' - ', SUM(lab_tra_total_amount_tmp-lab_tra_total_discount_tmp)) AS not_update_tra,lab_tra_cus_id_tmp FROM lab_test_tra_temp_tbl WHERE lab_tra_status_tmp = 'N' GROUP BY lab_tra_id_tmp ORDER BY lab_tra_id_tmp DESC",
    (error, results) => {
      if (error) {
        res.status(500).send(error);
      } else {
        if (results.length === 0) {
          // ? No data found for the specified
          res.status(204).json({ message: "No data found" });
        } else {
          res.status(200).json(results);
        }
      }
    }
  );
};

// ! get discount and total amounts selected transaction number
const getTotalValueByTraId = (req, res) => {
  const traID = req.params.id;

  // ? patient data get query
  db_con.query(
    "SELECT SUM(lab_tra_price_tmp) AS total_price, SUM(lab_tra_total_discount_tmp) AS total_discount, SUM(lab_tra_price_tmp-lab_tra_total_discount_tmp) AS total_amount,DATE_FORMAT(lab_tra_collect_datetime_tmp, '%Y-%m-%d %H:%i:%s') AS collected_time, lab_tra_doct_id_tmp AS doctor_id FROM lab_test_tra_temp_tbl WHERE lab_tra_id_tmp = ?",
    [traID],
    (error, results) => {
      if (error) {
        res.status(500).send(error);
      } else {
        if (results.length === 0) {
          // ? No data found
          res.status(404).json({ message: "No data found" });
        } else {
          res.status(200).json(results[0]);
        }
      }
    }
  );
};

// ! get all lab test transaction data for table view
const getAllLabTestDataTable = async (req, res) => {
  try {
    const { draw, start, length, search, order, columns } = req.query;
    const traID = req.params.id; // Extracting traID from parameters
    const page = Math.ceil(start / length) + 1;
    const limit = +length;
    const offset = (page - 1) * limit;

    const orderColumnIndex = order[0].column;
    const orderDirection = order[0].dir;
    const orderColumn = columns[orderColumnIndex].data;

    const searchPattern = `%${search.value}%`;
    let searchWhere = "";
    const searchParams = [];

    if (search.value) {
      searchWhere = " WHERE ";
      searchWhere += " (id_tmp LIKE ? ";
      searchWhere += " OR lab_tra_id_tmp LIKE ? ";
      searchWhere += " OR lab_tra_doct_id_tmp LIKE ? ";
      searchWhere += " OR lab_tra_collect_datetime_tmp LIKE ?) ";
      searchParams.push(
        searchPattern,
        searchPattern,
        searchPattern,
        searchPattern
      );
    }

    // Adding filter for specific lab_tra_id_tmp value (B01-LT00000001)
    if (traID) {
      searchWhere += searchWhere ? " AND " : " WHERE ";
      searchWhere += " lab_tra_id_tmp = ? ";
      searchParams.push(traID);
    }

    const selectQuery = `
      SELECT id_tmp, 
      lab_tra_id_tmp, 
      lab_tra_collect_datetime_tmp, 
      lab_tra_date_tmp, 
      lab_tra_time_tmp, 
      lab_tra_cus_id_tmp, 
      lab_tra_cus_acc_tmp, 
      lab_tra_doct_id_tmp, 
      lab_tra_doct_acc_tmp, 
      lab_tra_doct_rate_tmp, 
      lab_tra_lab_test_id_tmp, 
      lab_tra_price_tmp, 
      lab_tra_actual_price_tmp, 
      lab_tra_discount_rate_tmp, 
      lab_tra_total_discount_tmp, 
      lab_tra_total_amount_tmp, 
      lab_tra_total_amount_tmp - lab_tra_total_discount_tmp AS amount,
      lab_tra_ref_fee_tmp, 
      lab_tra_ref_fee_update_tmp, 
      lab_tra_hospital_id_tmp, 
      lab_tra_hospital_fee_tmp, 
      lab_tra_hospital_fee_update_tmp, 
      lab_tra_data_cal_status_tmp, 
      lab_tra_lab_test_send_tmp, 
      lab_tra_lab_test_send_time_tmp, 
      lab_tra_lab_test_receive_tmp, 
      lab_tra_lab_test_receive_time_tmp, 
      lab_tra_status_tmp, 
      lab_tra_branch_id_tmp, 
      lab_tra_user_id_tmp,
      lab_test_name,
      doct_name,
      hos_name
      FROM lab_test_tra_temp_tbl JOIN sys_lab_test_tbl ON lab_test_tra_temp_tbl.lab_tra_lab_test_id_tmp = sys_lab_test_tbl.lab_id
      JOIN sys_doctor_tbl ON lab_test_tra_temp_tbl.lab_tra_doct_id_tmp= sys_doctor_tbl.doct_id
      JOIN sys_hospital_tbl ON lab_test_tra_temp_tbl.lab_tra_hospital_id_tmp = sys_hospital_tbl.hos_id
      ${searchWhere}
      ORDER BY ${orderColumn} ${orderDirection}
      LIMIT ?, ?
    `;

    const countQuery = `
      SELECT COUNT(id_tmp) AS total 
      FROM lab_test_tra_temp_tbl JOIN sys_lab_test_tbl ON lab_test_tra_temp_tbl.lab_tra_lab_test_id_tmp = sys_lab_test_tbl.lab_id
      JOIN sys_doctor_tbl ON lab_test_tra_temp_tbl.lab_tra_doct_id_tmp= sys_doctor_tbl.doct_id
      JOIN sys_hospital_tbl ON lab_test_tra_temp_tbl.lab_tra_hospital_id_tmp = sys_hospital_tbl.hos_id
      ${searchWhere}
    `;

    db_con.query(
      selectQuery,
      [...searchParams, offset, limit],
      (error, results) => {
        if (error) {
          return res.status(500).send(error);
        }

        if (results.length === 0) {
          return res.status(200).json({
            draw: +draw,
            recordsTotal: 0,
            recordsFiltered: 0,
            data: [],
            message: "No data found in the table.",
          });
        }

        db_con.query(countQuery, searchParams, (countError, countResults) => {
          if (countError) {
            return res.status(500).send(countError);
          }

          const totalRecords = countResults[0].total;
          const response = {
            draw: +draw,
            recordsTotal: totalRecords,
            recordsFiltered: totalRecords,
            data: results,
          };

          res.status(200).json(response);
        });
      }
    );
  } catch (error) {
    res.status(500).send(error.message);
  }
};

const deleteLabTestTransactionByID = (req, res) => {
  const idToDelete = req.params.id;

  if (!idToDelete) {
    return res.status(400).send("ID is required");
  }

  const query = `DELETE FROM lab_test_tra_temp_tbl WHERE id_tmp = ?`;

  db_con.query(query, [idToDelete], (err, result) => {
    if (err) {
      console.error(err);
      return res.status(500).send("Error deleting data");
    }

    if (result.affectedRows > 0) {
      res.status(200).send({ message: "Deleted Successfully" });
    } else {
      res.status(404).send({ message: "No rows deleted" });
    }
  });
};

// ! get all lab test transaction data for table view
const getAfterAllLabTestDataTable = async (req, res) => {
  try {
    const { draw, start, length, search, order, columns } = req.query;
    const traID = req.params.id; // Extracting traID from parameters
    const page = Math.ceil(start / length) + 1;
    const limit = +length;
    const offset = (page - 1) * limit;

    const orderColumnIndex = order[0].column;
    const orderDirection = order[0].dir;
    const orderColumn = columns[orderColumnIndex].data;

    const searchPattern = `%${search.value}%`;
    let searchWhere = "";
    const searchParams = [];

    if (search.value) {
      searchWhere = " WHERE ";
      searchWhere += " (id LIKE ? ";
      searchWhere += " OR lab_tra_id LIKE ? ";
      searchWhere += " OR lab_tra_doct_id LIKE ? ";
      searchWhere += " OR lab_tra_collect_datetime LIKE ?) ";
      searchParams.push(
        searchPattern,
        searchPattern,
        searchPattern,
        searchPattern
      );
    }

    // Adding filter for specific lab_tra_id value (B01-LT00000001)
    if (traID) {
      searchWhere += searchWhere ? " AND " : " WHERE ";
      searchWhere += " lab_tra_id = ? ";
      searchParams.push(traID);
    }

    const selectQuery = `
      SELECT id, 
      lab_tra_id, 
      lab_tra_collect_datetime, 
      DATE_FORMAT(lab_tra_date, '%Y-%m-%d') AS lab_tra_date, 
      lab_tra_time, 
      lab_tra_cus_id, 
      lab_tra_cus_acc, 
      lab_tra_doct_id, 
      lab_tra_doct_acc, 
      lab_tra_doct_rate, 
      lab_tra_lab_test_id, 
      lab_tra_price, 
      lab_tra_actual_price, 
      lab_tra_discount_rate, 
      lab_tra_total_discount, 
      lab_tra_total_amount, 
      lab_tra_total_amount - lab_tra_total_discount AS amount,
      lab_tra_ref_fee, 
      lab_tra_ref_fee_update, 
      lab_tra_hospital_id, 
      lab_tra_hospital_fee, 
      lab_tra_hospital_fee_update, 
      lab_tra_data_cal_status, 
      lab_tra_lab_test_send, 
      lab_tra_lab_test_send_time, 
      lab_tra_lab_test_receive, 
      lab_tra_lab_test_receive_time, 
      lab_tra_status, 
      lab_tra_branch_id, 
      lab_tra_user_id,
      lab_test_name,
      doct_name,
      hos_name
      FROM lab_test_tra_tbl JOIN sys_lab_test_tbl ON lab_test_tra_tbl.lab_tra_lab_test_id = sys_lab_test_tbl.lab_id
      JOIN sys_doctor_tbl ON lab_test_tra_tbl.lab_tra_doct_id= sys_doctor_tbl.doct_id
      JOIN sys_hospital_tbl ON lab_test_tra_tbl.lab_tra_hospital_id = sys_hospital_tbl.hos_id
      JOIN sys_patient_tbl ON lab_test_tra_tbl.lab_tra_cus_id = sys_patient_tbl.pat_id
      ${searchWhere}
      ORDER BY ${orderColumn} ${orderDirection}
      LIMIT ?, ?
    `;

    const countQuery = `
      SELECT COUNT(id) AS total 
      FROM lab_test_tra_tbl JOIN sys_lab_test_tbl ON lab_test_tra_tbl.lab_tra_lab_test_id = sys_lab_test_tbl.lab_id
      JOIN sys_doctor_tbl ON lab_test_tra_tbl.lab_tra_doct_id= sys_doctor_tbl.doct_id
      JOIN sys_hospital_tbl ON lab_test_tra_tbl.lab_tra_hospital_id = sys_hospital_tbl.hos_id
      JOIN sys_patient_tbl ON lab_test_tra_tbl.lab_tra_cus_id = sys_patient_tbl.pat_id
      ${searchWhere}
    `;

    db_con.query(
      selectQuery,
      [...searchParams, offset, limit],
      (error, results) => {
        if (error) {
          return res.status(500).send(error);
        }

        if (results.length === 0) {
          return res.status(200).json({
            draw: +draw,
            recordsTotal: 0,
            recordsFiltered: 0,
            data: [],
            message: "No data found in the table.",
          });
        }

        db_con.query(countQuery, searchParams, (countError, countResults) => {
          if (countError) {
            return res.status(500).send(countError);
          }

          const totalRecords = countResults[0].total;
          const response = {
            draw: +draw,
            recordsTotal: totalRecords,
            recordsFiltered: totalRecords,
            data: results,
          };

          res.status(200).json(response);
        });
      }
    );
  } catch (error) {
    res.status(500).send(error.message);
  }
};

// ? get all lab test count summery
const getLabTestCountsSummeryByDate = (req, res) => {
  const dateTime = getCurrentDateTime();

  // Construct the SQL query using template literals
  const sqlQuery = `
    SELECT
      COUNT(lab_tra_id) AS total_count,
      COUNT(DISTINCT lab_tra_cus_id) AS customer_count,
      COUNT(CASE WHEN lab_tra_complete_status = 'Y' THEN lab_tra_id END) AS completed_count,
      COUNT(CASE WHEN lab_tra_complete_status = 'N' THEN lab_tra_id END) AS not_completed_count
    FROM
      lab_test_tra_tbl
    WHERE
      lab_tra_date = '${dateTime.date}';
  `;

  // Execute the SQL query
  db_con.query(sqlQuery, (error, results) => {
    if (error) {
      console.error(error);
      res.status(500).send(error); // Handle error response
    } else {
      res.status(200).json(results[0]); // Send JSON response with the lab test counts
    }
  });
};

// ? Define an API endpoint to get hourly lab test counts by date
const getLabTestHourlyCountsByDate = (req, res) => {
  // ? Get the current date and time using a function (Assuming getCurrentDateTime is defined elsewhere)
  const dateTime = getCurrentDateTime();

  // ? Construct the SQL query using a parameterized query
  const sqlQuery = `
    SELECT
      hours.hour,
      IFNULL(COUNT(ltt.lab_tra_id), 0) AS hourly_count
    FROM
      (SELECT 0 AS hour
       UNION SELECT 1 UNION SELECT 2 UNION SELECT 3
       UNION SELECT 4 UNION SELECT 5 UNION SELECT 6
       UNION SELECT 7 UNION SELECT 8 UNION SELECT 9
       UNION SELECT 10 UNION SELECT 11 UNION SELECT 12
       UNION SELECT 13 UNION SELECT 14 UNION SELECT 15
       UNION SELECT 16 UNION SELECT 17 UNION SELECT 18
       UNION SELECT 19 UNION SELECT 20 UNION SELECT 21
       UNION SELECT 22 UNION SELECT 23) AS hours
    LEFT JOIN
      lab_test_tra_tbl ltt ON hours.hour = HOUR(ltt.lab_tra_time) AND ltt.lab_tra_date = ?
    GROUP BY
      hours.hour
    ORDER BY
      hours.hour;
  `;

  // ? Execute the SQL query with parameters
  db_con.query(sqlQuery, [dateTime.date], (error, results) => {
    if (error) {
      // ? Log and handle any errors that occur during the database query
      console.error(error);
      res.status(500).send(error);
    } else {
      // ? If the query is successful, send the JSON response with the lab test hourly counts
      res.status(200).json(results);
    }
  });
};

// ! get all lab test transaction data for table view in doctor reference fee
const getDoctorRefFeeTestDataTable = async (req, res) => {
  try {
    const { draw, start, length, search, order, columns } = req.query;
    const refID = req.params.refId; // Extracting traID from parameters
    const traType = req.query.Type;
    const page = Math.ceil(start / length) + 1;
    const limit = +length;
    const offset = (page - 1) * limit;

    const orderColumnIndex = order[0].column;
    const orderDirection = order[0].dir;
    const orderColumn = columns[orderColumnIndex].data;

    const searchPattern = `%${search.value}%`;
    let searchWhere = "";
    const searchParams = [];

    // ? Check if the array of IDs is provided and not empty
    if (!traType || typeof traType !== "string" || traType.trim() === "") {
      return res
        .status(400)
        .json({ message: "Invalid or empty array of IDs provided" });
    }

    // ? Convert the string of IDs to an array
    const idArray = traType.split(",").map((id) => id.trim());

    // Optional: Remove any empty strings
    const nonEmptyIdArray = idArray.filter(Boolean);

    // ? Check if the array of IDs is valid
    if (
      !nonEmptyIdArray ||
      !Array.isArray(nonEmptyIdArray) ||
      nonEmptyIdArray.length === 0
    ) {
      return res
        .status(400)
        .json({ message: "Invalid or empty array of IDs provided" });
    }

    // ? Create a placeholder for the '?' in the SQL query based on the number of IDs
    const placeholders = nonEmptyIdArray.map(() => "?").join(",");

    if (search.value) {
      searchWhere = " WHERE ";
      searchWhere += " (id LIKE ? ";
      searchWhere += " OR lab_test_name LIKE ? ";
      searchWhere += " OR lab_tra_id LIKE ? ";
      searchWhere += " OR lab_tra_doct_id LIKE ? ";
      searchWhere += " OR lab_tra_collect_datetime LIKE ?) ";
      searchParams.push(
        searchPattern,
        searchPattern,
        searchPattern,
        searchPattern,
        searchPattern
      );
    }

    // ? Adding filter for specific lab_tra_id value (B01-LT00000001)
    if (refID) {
      searchWhere += searchWhere ? " AND " : " WHERE ";
      searchWhere += ` lab_tra_doct_id = ? AND lab_tra_ref_fee_update IN (${placeholders})`;
      searchParams.push(refID, ...nonEmptyIdArray);
    }

    const selectQuery = `
      SELECT id, 
      lab_tra_id, 
      lab_tra_collect_datetime, 
      DATE_FORMAT(lab_tra_date, '%Y-%m-%d %H:%i:%s') AS lab_tra_date,
      lab_tra_time, 
      lab_tra_cus_id, 
      lab_tra_cus_acc, 
      lab_tra_doct_id, 
      lab_tra_doct_acc, 
      lab_tra_doct_rate, 
      lab_tra_lab_test_id, 
      lab_tra_price, 
      lab_tra_actual_price, 
      lab_tra_discount_rate, 
      lab_tra_total_discount, 
      lab_tra_total_amount, 
      lab_tra_total_amount - lab_tra_total_discount AS amount,
      lab_tra_ref_fee, 
      lab_tra_ref_fee_update, 
      lab_tra_hospital_id, 
      lab_tra_hospital_fee, 
      lab_tra_hospital_fee_update, 
      lab_tra_data_cal_status, 
      lab_tra_lab_test_send, 
      DATE_FORMAT(lab_tra_lab_test_send_time, '%Y-%m-%d %H:%i:%s') AS lab_tra_lab_test_send_time, 
      lab_tra_lab_test_receive, 
      DATE_FORMAT(lab_tra_lab_test_receive_time, '%Y-%m-%d %H:%i:%s') AS lab_tra_lab_test_receive_time, 
      lab_tra_status, 
      lab_tra_branch_id, 
      lab_tra_user_id,
      lab_test_name,
      doct_name,
      hos_name,
      brach_name,
      DATE_FORMAT(lab_tra_ref_fee_pay_date, '%Y-%m-%d %H:%i:%s') AS lab_tra_ref_fee_pay_date,
      DATE_FORMAT(lab_tra_hospital_fee_pay_date, '%Y-%m-%d %H:%i:%s') AS lab_tra_hospital_fee_pay_date
      FROM lab_test_tra_tbl JOIN sys_lab_test_tbl ON lab_test_tra_tbl.lab_tra_lab_test_id = sys_lab_test_tbl.lab_id
      JOIN sys_doctor_tbl ON lab_test_tra_tbl.lab_tra_doct_id= sys_doctor_tbl.doct_id
      JOIN sys_hospital_tbl ON lab_test_tra_tbl.lab_tra_hospital_id = sys_hospital_tbl.hos_id 
      JOIN sys_branch_tbl ON lab_test_tra_tbl.lab_tra_branch_id= sys_branch_tbl.brach_id
      JOIN sys_patient_tbl ON lab_test_tra_tbl.lab_tra_cus_id = sys_patient_tbl.pat_id
      ${searchWhere}
      ORDER BY ${orderColumn} ${orderDirection}
      LIMIT ?, ?
    `;

    const countQuery = `
      SELECT COUNT(id) AS total 
      FROM lab_test_tra_tbl JOIN sys_lab_test_tbl ON lab_test_tra_tbl.lab_tra_lab_test_id = sys_lab_test_tbl.lab_id
      JOIN sys_doctor_tbl ON lab_test_tra_tbl.lab_tra_doct_id= sys_doctor_tbl.doct_id
      JOIN sys_hospital_tbl ON lab_test_tra_tbl.lab_tra_hospital_id = sys_hospital_tbl.hos_id
      JOIN sys_patient_tbl ON lab_test_tra_tbl.lab_tra_cus_id = sys_patient_tbl.pat_id
      ${searchWhere}
    `;

    db_con.query(
      selectQuery,
      [...searchParams, offset, limit],
      (error, results) => {
        if (error) {
          return res.status(500).send(error);
        }

        if (results.length === 0) {
          return res.status(200).json({
            draw: +draw,
            recordsTotal: 0,
            recordsFiltered: 0,
            data: [],
            message: "No data found in the table.",
          });
        }

        db_con.query(countQuery, searchParams, (countError, countResults) => {
          if (countError) {
            return res.status(500).send(countError);
          }

          const totalRecords = countResults[0].total;
          const response = {
            draw: +draw,
            recordsTotal: totalRecords,
            recordsFiltered: totalRecords,
            data: results,
          };

          res.status(200).json(response);
        });
      }
    );
  } catch (error) {
    res.status(500).send(error.message);
  }
};

// ! get all lab test transaction data for table view in hospital fee and transfer
const getHospitalFeeTransferTestDataTable = async (req, res) => {
  try {
    const { draw, start, length, search, order, columns } = req.query;
    const hospitalID = req.params.hospitalId; // Extracting traID from parameters
    const send_status = req.query.send_status;
    const receive_status = req.query.receive_status;
    const page = Math.ceil(start / length) + 1;
    const limit = +length;
    const offset = (page - 1) * limit;

    const orderColumnIndex = order[0].column;
    const orderDirection = order[0].dir;
    const orderColumn = columns[orderColumnIndex].data;

    const searchPattern = `%${search.value}%`;
    let searchWhere = "";
    const searchParams = [];

    if (
      !send_status ||
      typeof send_status !== "string" ||
      send_status.trim() === ""
    ) {
      return res
        .status(400)
        .json({ message: "Invalid or empty value for send_status provided" });
    }

    const sendStatusArray = send_status.split(",").map((id) => id.trim());
    const nonEmptySendStatusArray = sendStatusArray.filter(Boolean);

    if (
      !nonEmptySendStatusArray ||
      !Array.isArray(nonEmptySendStatusArray) ||
      nonEmptySendStatusArray.length === 0
    ) {
      return res.status(400).json({
        message: "Invalid or empty array of send_status values provided",
      });
    }

    const sendStatusPlaceholders = nonEmptySendStatusArray
      .map(() => "?")
      .join(",");

    if (
      !receive_status ||
      typeof receive_status !== "string" ||
      receive_status.trim() === ""
    ) {
      return res.status(400).json({
        message: "Invalid or empty value for receive_status provided",
      });
    }

    const receiveStatusArray = receive_status.split(",").map((id) => id.trim());
    const nonEmptyReceiveStatusArray = receiveStatusArray.filter(Boolean);

    if (
      !nonEmptyReceiveStatusArray ||
      !Array.isArray(nonEmptyReceiveStatusArray) ||
      nonEmptyReceiveStatusArray.length === 0
    ) {
      return res.status(400).json({
        message: "Invalid or empty array of receive_status values provided",
      });
    }

    const receiveStatusPlaceholders = nonEmptyReceiveStatusArray
      .map(() => "?")
      .join(",");

    if (search.value) {
      searchWhere = " WHERE ";
      searchWhere += " (id LIKE ? ";
      searchWhere += " OR lab_test_name LIKE ? ";
      searchWhere += " OR lab_tra_id LIKE ? ";
      searchWhere += " OR lab_tra_doct_id LIKE ? ";
      searchWhere += " OR pat_name LIKE ? ";
      searchWhere += " OR lab_tra_collect_datetime LIKE ?) ";
      searchParams.push(
        searchPattern,
        searchPattern,
        searchPattern,
        searchPattern,
        searchPattern,
        searchPattern
      );
    }

    // ? Adding filter for specific lab_tra_id value (B01-LT00000001)
    if (hospitalID) {
      searchWhere += searchWhere ? " AND " : " WHERE ";
      searchWhere += ` lab_tra_hospital_id = ? AND lab_tra_lab_test_send IN (${sendStatusPlaceholders}) AND lab_tra_lab_test_receive IN (${receiveStatusPlaceholders})`;
      searchParams.push(
        hospitalID,
        ...nonEmptySendStatusArray,
        ...nonEmptyReceiveStatusArray
      );
    }

    const selectQuery = `
      SELECT id, 
      lab_tra_id, 
      lab_tra_collect_datetime, 
      DATE_FORMAT(lab_tra_date, '%Y-%m-%d %H:%i:%s') AS lab_tra_date,
      lab_tra_time, 
      lab_tra_cus_id, 
      lab_tra_cus_acc,
      pat_name, 
      lab_tra_doct_id, 
      lab_tra_doct_acc, 
      lab_tra_doct_rate, 
      lab_tra_lab_test_id, 
      lab_tra_price, 
      lab_tra_actual_price, 
      lab_tra_discount_rate, 
      lab_tra_total_discount, 
      lab_tra_total_amount, 
      lab_tra_total_amount - lab_tra_total_discount AS amount,
      lab_tra_ref_fee, 
      lab_tra_ref_fee_update, 
      lab_tra_hospital_id, 
      lab_tra_hospital_fee, 
      lab_tra_hospital_fee_update, 
      lab_tra_data_cal_status, 
      lab_tra_lab_test_send, 
      DATE_FORMAT(lab_tra_lab_test_send_time, '%Y-%m-%d %H:%i:%s') AS lab_tra_lab_test_send_time, 
      lab_tra_lab_test_receive, 
      DATE_FORMAT(lab_tra_lab_test_receive_time, '%Y-%m-%d %H:%i:%s') AS lab_tra_lab_test_receive_time, 
      lab_tra_status, 
      lab_tra_branch_id, 
      lab_tra_user_id,
      lab_test_name,
      doct_name,
      hos_name,
      brach_name,
      DATE_FORMAT(lab_tra_ref_fee_pay_date, '%Y-%m-%d %H:%i:%s') AS lab_tra_ref_fee_pay_date,
      DATE_FORMAT(lab_tra_hospital_fee_pay_date, '%Y-%m-%d %H:%i:%s') AS lab_tra_hospital_fee_pay_date
      FROM lab_test_tra_tbl JOIN sys_lab_test_tbl ON lab_test_tra_tbl.lab_tra_lab_test_id = sys_lab_test_tbl.lab_id
      JOIN sys_doctor_tbl ON lab_test_tra_tbl.lab_tra_doct_id= sys_doctor_tbl.doct_id
      JOIN sys_hospital_tbl ON lab_test_tra_tbl.lab_tra_hospital_id = sys_hospital_tbl.hos_id 
      JOIN sys_patient_tbl ON lab_test_tra_tbl.lab_tra_cus_id = sys_patient_tbl.pat_id
      JOIN sys_branch_tbl ON (lab_test_tra_tbl.lab_tra_branch_id= sys_branch_tbl.brach_id)
      ${searchWhere}
      ORDER BY ${orderColumn} ${orderDirection}
      LIMIT ?, ?
    `;

    const countQuery = `
      SELECT COUNT(id) AS total 
      FROM lab_test_tra_tbl JOIN sys_lab_test_tbl ON lab_test_tra_tbl.lab_tra_lab_test_id = sys_lab_test_tbl.lab_id
      JOIN sys_doctor_tbl ON lab_test_tra_tbl.lab_tra_doct_id= sys_doctor_tbl.doct_id
      JOIN sys_hospital_tbl ON lab_test_tra_tbl.lab_tra_hospital_id = sys_hospital_tbl.hos_id
      JOIN sys_patient_tbl ON lab_test_tra_tbl.lab_tra_cus_id = sys_patient_tbl.pat_id
      ${searchWhere}
    `;

    db_con.query(
      selectQuery,
      [...searchParams, offset, limit],
      (error, results) => {
        if (error) {
          return res.status(500).send(error);
        }

        if (results.length === 0) {
          return res.status(200).json({
            draw: +draw,
            recordsTotal: 0,
            recordsFiltered: 0,
            data: [],
            message: "No data found in the table.",
          });
        }

        db_con.query(countQuery, searchParams, (countError, countResults) => {
          if (countError) {
            return res.status(500).send(countError);
          }

          const totalRecords = countResults[0].total;
          const response = {
            draw: +draw,
            recordsTotal: totalRecords,
            recordsFiltered: totalRecords,
            data: results,
          };

          res.status(200).json(response);
        });
      }
    );
  } catch (error) {
    res.status(500).send(error.message);
  }
};

// ? Modify the function to handle array input correctly
const getSumOfRefFeeByIds = (req, res) => {
  const ids = req.query.ids; // Assuming the array of IDs is sent as query parameters

  // ? Check if the array of IDs is provided and not empty
  if (!ids || typeof ids !== "string" || ids.trim() === "") {
    return res
      .status(400)
      .json({ message: "Invalid or empty array of IDs provided" });
  }

  // ? Convert the string of IDs to an array
  const idArray = ids.split(",").map((id) => parseInt(id.trim(), 10));

  // ? Check if the array of IDs is valid
  if (!idArray || !Array.isArray(idArray) || idArray.length === 0) {
    return res
      .status(400)
      .json({ message: "Invalid or empty array of IDs provided" });
  }

  // ? Create a placeholder for the '?' in the SQL query based on the number of IDs
  const placeholders = idArray.map(() => "?").join(",");

  // ? Construct the SQL query
  const sqlQuery = `SELECT SUM(lab_tra_ref_fee) AS total_ref_fee FROM lab_test_tra_tbl WHERE id IN (${placeholders})`;

  // ? Execute the query with the array of IDs
  db_con.query(sqlQuery, idArray, (error, results) => {
    if (error) {
      return res.status(500).send(error);
    } else {
      // ? The result will contain the sum of lab_tra_ref_fee for the provided IDs
      res.status(200).json({ total_ref_fee: results[0].total_ref_fee });
    }
  });
};

// ? calculate hospital fee for all payable
const getSumOfPayableHospitalFeeByHospitalId = (req, res) => {
  const hospital_id = req.params.hospitalId;

  // ? Construct the SQL query
  const sqlQuery = `SELECT IFNULL(SUM(lab_tra_hospital_fee), 0) AS total_payable_fee FROM lab_test_tra_tbl WHERE lab_tra_hospital_fee_update = 'P' AND lab_tra_hospital_id = ?`;

  // ? Execute the SQL query with parameters
  db_con.query(sqlQuery, hospital_id, (error, results) => {
    if (error) {
      // ? Log and handle any errors that occur during the database query
      console.error(error);
      res.status(500).send(error);
    } else {
      // ? If the query is successful, send the JSON response with the lab test hourly counts
      res.status(200).json({ total_payable_fee: results[0].total_payable_fee });
    }
  });
};

// ? calculate hospital fee for all payable
const getSumOfPayableHospitalFeeBySelectedId = (req, res) => {
  const ids = req.query.ids;

  // ? Check if the array of IDs is provided and not empty
  if (!ids || typeof ids !== "string" || ids.trim() === "") {
    return res
      .status(400)
      .json({ message: "Invalid or empty array of IDs provided" });
  }

  // ? Convert the string of IDs to an array
  const idArray = ids.split(",").map((id) => parseInt(id.trim(), 10));

  // ? Check if the array of IDs is valid
  if (!idArray || !Array.isArray(idArray) || idArray.length === 0) {
    return res
      .status(400)
      .json({ message: "Invalid or empty array of IDs provided" });
  }

  // ? Construct the SQL query using parameterized query
  const sqlQuery =
    "SELECT IFNULL(SUM(lab_tra_hospital_fee), 0) AS total_payable_fee FROM lab_test_tra_tbl WHERE id IN (?)";

  // ? Execute the SQL query with parameters
  db_con.query(sqlQuery, [idArray], (error, results) => {
    if (error) {
      // ? Log and handle any errors that occur during the database query
      console.error(error);
      res.status(500).send(error);
    } else {
      // ? If the query is successful, send the JSON response with the lab test hourly counts

      res.status(200).json({ total_payable_fee: results[0].total_payable_fee });
    }
  });
};

const createPatientReportUploadFolder = (req, res) => {
  const folder_name = req.params.folder_name;

  const newDirectoryPath = "../Report-Print/Document/" + folder_name;

  // Check if the directory already exists
  if (!fs.existsSync(newDirectoryPath)) {
    // Create the directory
    fs.mkdir(newDirectoryPath, (err) => {
      if (err) {
        console.error("Error creating directory:", err);
        res.status(500).send("Error creating directory");
      } else {
        console.log("Directory created successfully");
        res.send("Directory created successfully");
      }
    });
  } else {
    console.log("Directory already exists");
    res.send("Directory already exists");
  }
};

const updateReceivedTestTransferStatus = (req, res) => {
  const dateTime = getCurrentDateTime();
  const reference_id = req.params.id;

  // ? update lab test tra table doctor ref fee update status
  const lab_tra_update_query = `
  UPDATE lab_test_tra_tbl
  SET lab_tra_lab_test_receive = 'Y', lab_tra_complete_status = 'Y', lab_tra_lab_test_receive_time = ?, lab_tra_complete_datetime = ?
  WHERE id = ?`;

  db_con.query(
    lab_tra_update_query,
    [
      dateTime.date + " " + dateTime.time,
      dateTime.date + " " + dateTime.time,
      reference_id,
    ],
    (error, results) => {
      if (error) {
        console.error("Error updating lab_test_tra_tbl:", error);
        res.status(500).json({ error: "Internal Server Error" });
      } else {
        // Check if any rows were affected by the update
        if (results.affectedRows > 0) {
          res.status(200).json({ message: "Update successful" });
        } else {
          res.status(404).json({ error: "Record not found" });
        }
      }
    }
  );
};

const getAllDataTestTransactionById = (req, res) => {
  const labTransactionID = req.params.id;

  db_con.query(
    `SELECT
    id,
    lab_tra_id,
    DATE_FORMAT(lab_tra_collect_datetime, '%Y-%m-%d %H:%i:%s') AS lab_tra_collect_datetime,
    DATE_FORMAT(lab_tra_date, '%Y-%m-%d') AS lab_tra_date,
    DATE_FORMAT(lab_tra_time, '%H:%i:%s') AS lab_tra_time,
    lab_tra_cus_id,
    pat_name,
    pat_age,
    pat_gender,
    lab_tra_doct_id,
    doct_name,
    lab_tra_lab_test_id,
    lab_test_name,
    lab_tra_price,
    lab_tra_actual_price,
    lab_tra_discount_rate,
    lab_tra_total_discount,
    lab_tra_total_amount,
    lab_tra_ref_fee,
    lab_tra_ref_fee_update,
    lab_tra_hospital_id,
    hos_name,
    lab_tra_hospital_fee,
    lab_tra_hospital_fee_update,
    lab_tra_data_cal_status,
    lab_tra_lab_test_send,
    DATE_FORMAT(lab_tra_lab_test_send_time, '%Y-%m-%d %H:%i:%s') AS lab_tra_lab_test_send_time,
    lab_tra_lab_test_receive,
    DATE_FORMAT(lab_tra_lab_test_receive_time, '%Y-%m-%d %H:%i:%s') AS lab_tra_lab_test_receive_time,
    lab_tra_status,
    lab_tra_branch_id,
    lab_tra_user_id,
    user_fname,
    brach_name,
    lab_tra_complete_status,
    DATE_FORMAT(lab_tra_ref_fee_pay_date, '%Y-%m-%d %H:%i:%s') AS lab_tra_ref_fee_pay_date,
    DATE_FORMAT(lab_tra_hospital_fee_pay_date, '%Y-%m-%d %H:%i:%s') AS lab_tra_hospital_fee_pay_date,
    lab_tra_hospital_fee_pay_tra
  FROM
    lab_test_tra_tbl
    INNER JOIN sys_doctor_tbl
      ON (
        lab_test_tra_tbl.lab_tra_doct_id = sys_doctor_tbl.doct_id
      )
    INNER JOIN sys_user_tbl
      ON (
        lab_test_tra_tbl.lab_tra_user_id = sys_user_tbl.user_id
      )
    INNER JOIN sys_patient_tbl
      ON (
        lab_test_tra_tbl.lab_tra_cus_id = sys_patient_tbl.pat_id
      )
    INNER JOIN sys_hospital_tbl
      ON (
        lab_test_tra_tbl.lab_tra_hospital_id = sys_hospital_tbl.hos_id
      )
    INNER JOIN sys_lab_test_tbl
      ON (
        lab_test_tra_tbl.lab_tra_lab_test_id = sys_lab_test_tbl.lab_id
      )
    INNER JOIN sys_branch_tbl
      ON (
        lab_test_tra_tbl.lab_tra_branch_id = sys_branch_tbl.brach_id
      )
  WHERE lab_test_tra_tbl.id = ?`,
    [labTransactionID],
    (error, results) => {
      if (error) {
        res.status(500).send(error);
      } else {
        if (results.length === 0) {
          res.status(404).json({
            message: "No data found for the specified lab transaction ID",
          });
        } else {
          res.status(200).json(results[0]);
        }
      }
    }
  );
};

module.exports = {
  insertLabTestTransaction,
  getAllNotUpdateTransactions,
  getTotalValueByTraId,
  getAllLabTestDataTable,
  deleteLabTestTransactionByID,
  getAfterAllLabTestDataTable,
  getLabTestCountsSummeryByDate,
  getLabTestHourlyCountsByDate,
  getDoctorRefFeeTestDataTable,
  getSumOfRefFeeByIds,
  getHospitalFeeTransferTestDataTable,
  getSumOfPayableHospitalFeeByHospitalId,
  getSumOfPayableHospitalFeeBySelectedId,
  createPatientReportUploadFolder,
  updateReceivedTestTransferStatus,
  getAllDataTestTransactionById,
};
