const db_con = require("../../../config/database-connection");

const currentDate = new Date();

const formattedDate = currentDate.toLocaleString("en-US", {
  year: "numeric",
  month: "2-digit",
  day: "2-digit",
});

// Extracting date components and formatting as "YYYY-MM-DD"
const [month, day, year] = formattedDate.split("/");
const Today_Date = `${year}-${month}-${day}`;

const Today_Time = currentDate.toLocaleString("en-US", {
  hour: "2-digit",
  minute: "2-digit",
  second: "2-digit",
  hour12: false, // Use 24-hour format
});

// ! Create lab test data query
const createLabTest = (req, res) => {
  const {
    lab_test_name,
    lab_test_short_name,
    lab_test_code,
    lab_test_price,
    lab_test_com_rate,
    lab_test_active,
    lab_test_type,
    lab_test_hospital_id,
    lab_test_variasion,
    lab_test_print_name,
    lab_test_analyzer,
  } = req.body;

  // Calculate lab_test_profit
  const lab_test_profit = (lab_test_price * lab_test_com_rate) / 100;

  // Validate required fields
  if (
    !lab_test_name ||
    !lab_test_price ||
    !lab_test_com_rate ||
    !lab_test_analyzer ||
    !lab_test_print_name
  ) {
    return res.status(400).json({ error: "All fields are required" });
  }

  // Check if the lab test name already exists
  const queryCheckName =
    "SELECT COUNT(*) AS count FROM sys_lab_test_tbl WHERE lab_test_name = ?";
  db_con.query(
    queryCheckName,
    [lab_test_name],
    (checkNameError, checkNameResults) => {
      if (checkNameError) {
        return res.status(500).send(checkNameError);
      }

      const nameExists = checkNameResults[0].count > 0;

      if (nameExists) {
        return res.status(400).json({ error: "Lab test name already exists" });
      }

      // Get the max from existing lab tests
      const queryMaxId =
        "SELECT MAX(CAST(SUBSTRING(lab_id,4) AS UNSIGNED)) AS max_id FROM sys_lab_test_tbl WHERE lab_test_type = ? AND lab_test_variasion = ?";
      db_con.query(
        queryMaxId,
        [lab_test_type, lab_test_variasion],
        (maxIdError, results) => {
          if (maxIdError) {
            return res.status(500).send(maxIdError);
          }

          const maxID = results[0].max_id || 0;
          const nextID = maxID + 1;

          // Format the next lab test id
          const labTestId =
            lab_test_type +
            lab_test_variasion +
            "-" +
            `${String(nextID).padStart(5, "0")}`;

          // Insert lab test data
          const queryInsert = `
            INSERT INTO sys_lab_test_tbl 
              (lab_id, lab_test_name, lab_test_short_name, lab_test_code, lab_test_price, 
              lab_test_com_rate, lab_test_profit, lab_test_create_date, 
              lab_test_update_date, lab_test_active, lab_test_type, lab_test_hospital_id, 
              lab_test_variasion,lab_test_print_name,lab_test_analyzer) 
            VALUES (?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?)
          `;
          const values = [
            labTestId,
            lab_test_name,
            lab_test_short_name,
            labTestId,
            lab_test_price,
            lab_test_com_rate,
            lab_test_profit,
            `${Today_Date}` + " " + `${Today_Time}`,
            `${Today_Date}` + " " + `${Today_Time}`,
            lab_test_active,
            lab_test_type,
            lab_test_hospital_id,
            lab_test_variasion,
            lab_test_print_name,
            lab_test_analyzer,
          ];

          db_con.query(queryInsert, values, (insertError, insertResults) => {
            if (insertError) {
              res.status(500).send(insertError);
            } else {
              res.status(201).json({
                message: "Lab test added successfully",
                labTestId: labTestId,
              });
            }
          });
        }
      );
    }
  );
};

// ! Update lab test data query
const updateLabTest = (req, res) => {
  const labTestId = req.params.id;
  const {
    lab_test_name,
    lab_test_short_name,
    lab_test_code,
    lab_test_price,
    lab_test_com_rate,
    lab_test_active,
    lab_test_type,
    lab_test_hospital_id,
    lab_test_variasion,
    lab_test_print_name,
    lab_test_analyzer,
  } = req.body;

  // Calculate lab_test_profit
  const lab_test_profit = (lab_test_price * lab_test_com_rate) / 100;

  // Update lab test details
  const queryUpdate = `
      UPDATE sys_lab_test_tbl 
      SET 
        lab_test_name = ?, 
        lab_test_short_name = ?, 
        lab_test_code = ?, 
        lab_test_price = ?, 
        lab_test_com_rate = ?, 
        lab_test_profit = ?, 
        lab_test_update_date = NOW(), 
        lab_test_active = ?, 
        lab_test_type = ?, 
        lab_test_hospital_id = ?,
        lab_test_variasion = ?,
        lab_test_print_name = ?,
        lab_test_analyzer = ? 
      WHERE lab_id = ?
    `;
  const values = [
    lab_test_name,
    lab_test_short_name,
    lab_test_code,
    lab_test_price,
    lab_test_com_rate,
    lab_test_profit,
    lab_test_active,
    lab_test_type,
    lab_test_hospital_id,
    lab_test_variasion,
    lab_test_print_name,
    lab_test_analyzer,
    labTestId,
  ];

  db_con.query(queryUpdate, values, (error, results) => {
    if (error) {
      res.status(500).send(error);
    } else {
      if (results.affectedRows > 0) {
        res.status(200).json({ message: "Lab test updated successfully" });
      } else {
        res.status(404).json({ message: "Lab test not found" });
      }
    }
  });
};

// ! Get all lab test data query (table view)
const getAllLabTests = (req, res) => {
  try {
    const { draw, start, length, search, order, columns } = req.query;
    const page = Math.ceil(start / length) + 1;
    const limit = +length;
    const offset = (page - 1) * limit;

    const orderColumnIndex = order[0].column;
    const orderDirection = order[0].dir;
    const orderColumn = columns[orderColumnIndex].data;

    const searchPattern = `%${search.value}%`;

    const selectQuery = `
        SELECT 
          lab_id, 
          lab_test_name, 
          lab_test_short_name, 
          lab_test_code, 
          lab_test_price, 
          lab_test_com_rate, 
          lab_test_profit, 
          lab_test_create_date, 
          lab_test_update_date, 
          lab_test_active, 
          lab_test_type, 
          lab_test_hospital_id,
          lab_test_variasion,
          lab_test_print_name,
          lab_test_analyzer,
          hos_name
        FROM sys_lab_test_tbl JOIN sys_hospital_tbl ON sys_lab_test_tbl.lab_test_hospital_id = sys_hospital_tbl.hos_id
        WHERE 
          lab_test_name LIKE ? OR 
          lab_test_short_name LIKE ? OR 
          lab_test_code LIKE ? OR 
          lab_test_type LIKE ? OR
          lab_test_print_name LIKE ? OR
          lab_test_analyzer LIKE ?
        ORDER BY ${orderColumn} ${orderDirection}
        LIMIT ?, ?
      `;

    const countQuery = `
        SELECT COUNT(lab_id) AS total FROM sys_lab_test_tbl JOIN sys_hospital_tbl ON sys_lab_test_tbl.lab_test_hospital_id = sys_hospital_tbl.hos_id
        WHERE 
          lab_test_name LIKE ? OR 
          lab_test_short_name LIKE ? OR 
          lab_test_code LIKE ? OR 
          lab_test_type LIKE ? OR
          lab_test_print_name LIKE ? OR
          lab_test_analyzer LIKE ?
      `;

    db_con.query(
      selectQuery,
      [
        searchPattern,
        searchPattern,
        searchPattern,
        searchPattern,
        searchPattern,
        searchPattern,
        offset,
        limit,
      ],
      (error, results) => {
        if (error) {
          return res.status(500).send(error);
        }

        // Check if there's no data in the table
        if (results.length === 0) {
          return res.status(200).json({
            draw: +draw,
            recordsTotal: 0,
            recordsFiltered: 0,
            data: [],
            message: "No data found in the table.",
          });
        }

        db_con.query(
          countQuery,
          [
            searchPattern,
            searchPattern,
            searchPattern,
            searchPattern,
            searchPattern,
            searchPattern,
          ],
          (countError, countResults) => {
            if (countError) {
              return res.status(500).send(countError);
            }

            const totalRecords = countResults[0].total;
            const response = {
              draw: +draw,
              recordsTotal: totalRecords,
              recordsFiltered: totalRecords, // Use totalRecords for recordsFiltered
              data: results,
            };

            res.status(200).json(response);
          }
        );
      }
    );
  } catch (error) {
    res.status(500).send(error.message);
  }
};

// ! Get lab test data query selected lab test id
const getLabTestById = (req, res) => {
  const labTestId = req.params.id;

  // Get lab test data query
  const querySelect = `
      SELECT 
        lab_id, 
        lab_test_name, 
        lab_test_short_name, 
        lab_test_code, 
        lab_test_price, 
        lab_test_com_rate, 
        lab_test_profit, 
        lab_test_create_date, 
        lab_test_update_date, 
        lab_test_active, 
        lab_test_type, 
        lab_test_hospital_id,
        lab_test_variasion,
        lab_test_print_name,
        lab_test_analyzer,
        hos_name
      FROM sys_lab_test_tbl JOIN sys_hospital_tbl ON sys_lab_test_tbl.lab_test_hospital_id = sys_hospital_tbl.hos_id
      WHERE lab_id = ?
    `;

  db_con.query(querySelect, [labTestId], (error, results) => {
    if (error) {
      res.status(500).send(error);
    } else {
      if (results.length === 0) {
        // No data found for the specified lab test ID
        res.status(404).json({
          message: "No data found for the specified lab test ID",
        });
      } else {
        res.status(200).json(results[0]);
      }
    }
  });
};

// ! Get all lab test name query selected
const getAllLabTestName = (req, res) => {
  const labTestId = req.params.id;

  // Get lab test data query
  const querySelect = `
      SELECT 
        lab_test_name
      FROM sys_lab_test_tbl
    `;

  db_con.query(querySelect, (error, results) => {
    if (error) {
      res.status(500).send(error);
    } else {
      if (results.length === 0) {
        // No data found for the specified lab test ID
        res.status(404).json({
          message: "No data found",
        });
      } else {
        res.status(200).json(results);
      }
    }
  });
};

// ! get all Lab Test names and id query
const getLabTestNamesAndID = (req, res) => {
  // ? LabTest data get query
  db_con.query(
    "SELECT lab_id,lab_test_name FROM sys_lab_test_tbl",
    (error, results) => {
      if (error) {
        res.status(500).send(error);
      } else {
        if (results.length === 0) {
          // ? No data found for the specified LabTest ID
          res
            .status(404)
            .json({ message: "No data found for the specified lab test ID" });
        } else {
          res.status(200).json(results);
        }
      }
    }
  );
};

// ! get all Lab Test names and id query
const getLabTestNamesAndIDOwnLabTest = (req, res) => {
  // ? LabTest data get query
  db_con.query(
    "SELECT lab_id,lab_test_name FROM sys_lab_test_tbl WHERE lab_test_type = 'A'",
    (error, results) => {
      if (error) {
        res.status(500).send(error);
      } else {
        if (results.length === 0) {
          // ? No data found for the specified LabTest ID
          res
            .status(404)
            .json({ message: "No data found for the specified lab test ID" });
        } else {
          res.status(200).json(results);
        }
      }
    }
  );
};

// ? view lab test by hospital id
const getAllLabTestByHospitalId = async (req, res) => {
  try {
    const { draw, start, length, search, order, columns } = req.query;
    const hospital_id = req.params.hospital_id;
    const page = Math.ceil(start / length) + 1;
    const limit = +length;
    const offset = (page - 1) * limit;

    const orderColumnIndex = order[0].column;
    const orderDirection = order[0].dir;
    const orderColumn = columns[orderColumnIndex].data;

    const searchPattern = `%${search.value}%`;
    let searchWhere = "";
    const searchParams = [];

    if (search.value) {
      searchWhere = " WHERE ";
      searchWhere += " (lab_test_name LIKE ? ";
      searchWhere += " OR lab_id LIKE ? ";
      searchWhere += " OR lab_test_code LIKE ? ";
      searchWhere += " OR lab_test_code LIKE ? ";
      searchWhere += " OR lab_test_print_name LIKE ? ";
      searchWhere += " OR lab_test_analyzer LIKE ? ) ";
      searchParams.push(
        searchPattern,
        searchPattern,
        searchPattern,
        searchPattern,
        searchPattern,
        searchPattern
      );
    }

    if (hospital_id) {
      searchWhere += searchWhere ? " AND " : " WHERE ";
      searchWhere += `lab_test_hospital_id = ?`;
      searchParams.push(hospital_id);
    }

    const selectQuery = `
    SELECT
          lab_id, 
          lab_test_name, 
          lab_test_short_name, 
          lab_test_code, 
          lab_test_price, 
          lab_test_com_rate, 
          lab_test_profit, 
          lab_test_create_date, 
          lab_test_update_date, 
          lab_test_active, 
          lab_test_type, 
          lab_test_hospital_id,
          lab_test_variasion,
          lab_test_print_name,
          lab_test_analyzer,
          hos_name
    FROM sys_lab_test_tbl 
    JOIN sys_hospital_tbl ON sys_lab_test_tbl.lab_test_hospital_id = sys_hospital_tbl.hos_id
    ${searchWhere}
    ORDER BY ${orderColumn} ${orderDirection}
    LIMIT ?, ?
    `;

    const countQuery = `
    SELECT COUNT(lab_id) AS total 
    FROM sys_lab_test_tbl 
    JOIN sys_hospital_tbl ON sys_lab_test_tbl.lab_test_hospital_id = sys_hospital_tbl.hos_id
    ${searchWhere};
      `;

    db_con.query(
      selectQuery,
      [...searchParams, offset, limit],
      (error, results) => {
        if (error) {
          return res.status(500).send(error);
        }

        if (results.length === 0) {
          return res.status(200).json({
            draw: +draw,
            recordsTotal: 0,
            recordsFiltered: 0,
            data: [],
            message: "No data found in the table.",
          });
        }

        db_con.query(countQuery, searchParams, (countError, countResults) => {
          if (countError) {
            return res.status(500).send(countError);
          }

          const totalRecords = countResults[0].total;
          const response = {
            draw: +draw,
            recordsTotal: totalRecords,
            recordsFiltered: totalRecords,
            data: results,
          };

          res.status(200).json(response);
        });
      }
    );
  } catch (error) {
    res.status(500).send(error.message);
  }
};

// ! Update lab test price
const updateLabTestPrice = (req, res) => {
  const labTestId = req.params.id;
  const { lab_test_price } = req.body;

  // ? SQL query to update lab test details
  const queryUpdate = `
    UPDATE sys_lab_test_tbl 
    SET 
      lab_test_price = ?,
      lab_test_update_date = NOW()
    WHERE lab_id = ?
  `;

  // ? Values for the UPDATE query
  const values = [lab_test_price, labTestId];

  // ? Call the stored procedure
  const callProcedure = `CALL SP_actual_price_update(?)`;

  // ? Execute the UPDATE query
  db_con.query(queryUpdate, values, (error, results) => {
    if (error) {
      // ? Send error response if UPDATE query fails
      res.status(500).send(error);
    } else {
      if (results.affectedRows > 0) {
        // ? If UPDATE is successful, call the stored procedure
        db_con.query(callProcedure, [labTestId], (procError) => {
          if (procError) {
            // ? Send error response if stored procedure fails
            res.status(500).send(procError);
          } else {
            // ? Send success response if everything is successful
            res.status(200).json({
              message: "Lab test price and profit updated successfully",
            });
          }
        });
      } else {
        // ? If no rows were affected by UPDATE, send not found response
        res.status(404).json({ message: "Lab test not found" });
      }
    }
  });
};

// ! create lab test result worksheet
const createLabTestWorkSheet = (req, res) => {
  const {
    wk_lab_test_id,
    wk_test_sub_result_name,
    wk_test_sub_topic,
    wk_test_units,
    wk_test_default_range,
    wk_test_default_value,
    wk_test_flug_range_min_female,
    wk_test_flug_range_max_female,
    wk_test_flug_range_min_male,
    wk_test_flug_range_max_male,
    wk_test_ref_range_topic_1,
    wk_test_ref_range_name_1,
    wk_test_ref_range_1,
    wk_test_ref_range_topic_2,
    wk_test_ref_range_name_2,
    wk_test_ref_range_2,
    wk_test_ref_range_topic_3,
    wk_test_ref_range_name_3,
    wk_test_ref_range_3,
    wk_test_ref_range_topic_4,
    wk_test_ref_range_name_4,
    wk_test_ref_range_4,
    wk_test_ref_range_topic_5,
    wk_test_ref_range_name_5,
    wk_test_ref_range_5,
    wk_test_ref_range_topic_6,
    wk_test_ref_range_name_6,
    wk_test_ref_range_6,
    wk_test_ref_range_topic_7,
    wk_test_ref_range_name_7,
    wk_test_ref_range_7,
    wk_test_ref_range_topic_8,
    wk_test_ref_range_name_8,
    wk_test_ref_range_8,
    wk_test_ref_range_topic_9,
    wk_test_ref_range_name_9,
    wk_test_ref_range_9,
    wk_test_ref_range_topic_10,
    wk_test_ref_range_name_10,
    wk_test_ref_range_10,
    wk_test_ref_range_topic_11,
    wk_test_ref_range_name_11,
    wk_test_ref_range_11,
    wk_test_ref_range_topic_12,
    wk_test_ref_range_name_12,
    wk_test_ref_range_12,
    wk_test_ref_range_topic_13,
    wk_test_ref_range_name_13,
    wk_test_ref_range_13,
    wk_test_ref_range_topic_14,
    wk_test_ref_range_name_14,
    wk_test_ref_range_14,
    wk_test_ref_range_show_status,
  } = req.body;

  // Get the max from existing lab tests
  const queryMaxId =
    "SELECT MAX(CAST(SUBSTRING(wk_test_sub_result_id,2) AS UNSIGNED)) AS max_id FROM sys_lab_test_work_sheet_tbl";

  db_con.query(queryMaxId, (maxIdError, results) => {
    if (maxIdError) {
      return res.status(500).send(maxIdError);
    }

    const maxID = results[0].max_id || 0;
    const nextID = maxID + 1;

    // Format the next lab test id
    const wk_test_sub_result_id = "R" + `${String(nextID).padStart(5, "0")}`;

    // Insert lab test data
    const queryInsert = `
      INSERT INTO sys_lab_test_work_sheet_tbl 
      (wk_lab_test_id, wk_test_sub_result_id, wk_test_sub_result_name, wk_test_sub_topic, 
      wk_test_units, wk_test_default_range, wk_test_default_value, 
      wk_test_flug_range_min_female, wk_test_flug_range_max_female,
      wk_test_flug_range_min_male, wk_test_flug_range_max_male,
      wk_test_ref_range_topic_1, wk_test_ref_range_name_1, wk_test_ref_range_1, 
      wk_test_ref_range_topic_2, wk_test_ref_range_name_2, wk_test_ref_range_2, 
      wk_test_ref_range_topic_3, wk_test_ref_range_name_3, wk_test_ref_range_3, 
      wk_test_ref_range_topic_4, wk_test_ref_range_name_4, wk_test_ref_range_4, 
      wk_test_ref_range_topic_5, wk_test_ref_range_name_5, wk_test_ref_range_5, 
      wk_test_ref_range_topic_6, wk_test_ref_range_name_6, wk_test_ref_range_6, 
      wk_test_ref_range_topic_7, wk_test_ref_range_name_7, wk_test_ref_range_7, 
      wk_test_ref_range_topic_8, wk_test_ref_range_name_8, wk_test_ref_range_8, 
      wk_test_ref_range_topic_9, wk_test_ref_range_name_9, wk_test_ref_range_9, 
      wk_test_ref_range_topic_10, wk_test_ref_range_name_10, wk_test_ref_range_10, 
      wk_test_ref_range_topic_11, wk_test_ref_range_name_11, wk_test_ref_range_11, 
      wk_test_ref_range_topic_12, wk_test_ref_range_name_12, wk_test_ref_range_12, 
      wk_test_ref_range_topic_13, wk_test_ref_range_name_13, wk_test_ref_range_13, 
      wk_test_ref_range_topic_14, wk_test_ref_range_name_14, wk_test_ref_range_14, 
      wk_test_ref_range_show_status) 
      VALUES (?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?)
    `;

    const values = [
      wk_lab_test_id,
      wk_test_sub_result_id,
      wk_test_sub_result_name,
      wk_test_sub_topic,
      wk_test_units,
      wk_test_default_range,
      wk_test_default_value,
      wk_test_flug_range_min_female,
      wk_test_flug_range_max_female,
      wk_test_flug_range_min_male,
      wk_test_flug_range_max_male,
      wk_test_ref_range_topic_1,
      wk_test_ref_range_name_1,
      wk_test_ref_range_1,
      wk_test_ref_range_topic_2,
      wk_test_ref_range_name_2,
      wk_test_ref_range_2,
      wk_test_ref_range_topic_3,
      wk_test_ref_range_name_3,
      wk_test_ref_range_3,
      wk_test_ref_range_topic_4,
      wk_test_ref_range_name_4,
      wk_test_ref_range_4,
      wk_test_ref_range_topic_5,
      wk_test_ref_range_name_5,
      wk_test_ref_range_5,
      wk_test_ref_range_topic_6,
      wk_test_ref_range_name_6,
      wk_test_ref_range_6,
      wk_test_ref_range_topic_7,
      wk_test_ref_range_name_7,
      wk_test_ref_range_7,
      wk_test_ref_range_topic_8,
      wk_test_ref_range_name_8,
      wk_test_ref_range_8,
      wk_test_ref_range_topic_9,
      wk_test_ref_range_name_9,
      wk_test_ref_range_9,
      wk_test_ref_range_topic_10,
      wk_test_ref_range_name_10,
      wk_test_ref_range_10,
      wk_test_ref_range_topic_11,
      wk_test_ref_range_name_11,
      wk_test_ref_range_11,
      wk_test_ref_range_topic_12,
      wk_test_ref_range_name_12,
      wk_test_ref_range_12,
      wk_test_ref_range_topic_13,
      wk_test_ref_range_name_13,
      wk_test_ref_range_13,
      wk_test_ref_range_topic_14,
      wk_test_ref_range_name_14,
      wk_test_ref_range_14,
      wk_test_ref_range_show_status,
    ];

    db_con.query(queryInsert, values, (insertError, insertResults) => {
      if (insertError) {
        res.status(500).send(insertError);
      } else {
        res.status(201).json({
          message: "Lab test worksheet created successfully",
          wk_test_sub_result_id: wk_test_sub_result_id,
        });
      }
    });
  });
};

// ! view lab test worksheet data by lab test id
const getAllLabTestWorksheetDataByLabTestId = async (req, res) => {
  try {
    const { draw, start, length, search, order, columns } = req.query;
    const lab_test_id = req.params.lab_test_id;
    const page = Math.ceil(start / length) + 1;
    const limit = +length;
    const offset = (page - 1) * limit;

    const orderColumnIndex = order[0].column;
    const orderDirection = order[0].dir;
    const orderColumn = columns[orderColumnIndex].data;

    const searchPattern = `%${search.value}%`;
    let searchWhere = "";
    const searchParams = [];

    if (search.value) {
      searchWhere = " WHERE ";
      searchWhere += " (wk_test_sub_result_name LIKE ? ";
      searchWhere += " OR wk_test_sub_topic LIKE ? ) ";
      searchParams.push(searchPattern, searchPattern);
    }

    if (lab_test_id) {
      searchWhere += searchWhere ? " AND " : " WHERE ";
      searchWhere += `wk_lab_test_id = ?`;
      searchParams.push(lab_test_id);
    }

    const selectQuery = `
    SELECT
          *
    FROM sys_lab_test_work_sheet_tbl
    ${searchWhere}
    ORDER BY ${orderColumn} ${orderDirection}
    LIMIT ?, ?
    `;

    const countQuery = `
    SELECT COUNT(wk_test_sub_result_id) AS total 
    FROM sys_lab_test_work_sheet_tbl
    ${searchWhere};
      `;

    db_con.query(
      selectQuery,
      [...searchParams, offset, limit],
      (error, results) => {
        if (error) {
          return res.status(500).send(error);
        }

        if (results.length === 0) {
          return res.status(200).json({
            draw: +draw,
            recordsTotal: 0,
            recordsFiltered: 0,
            data: [],
            message: "No data found in the table.",
          });
        }

        db_con.query(countQuery, searchParams, (countError, countResults) => {
          if (countError) {
            return res.status(500).send(countError);
          }

          const totalRecords = countResults[0].total;
          const response = {
            draw: +draw,
            recordsTotal: totalRecords,
            recordsFiltered: totalRecords,
            data: results,
          };

          res.status(200).json(response);
        });
      }
    );
  } catch (error) {
    res.status(500).send(error.message);
  }
};

// ! Get lab test worksheet data by worksheet sub result id
const getLabTestWorksheetDataBySubResultId = (req, res) => {
  const result_sheet_data_id = req.params.id;

  // Get lab test data query
  const querySelect = `
      SELECT 
        *
      FROM sys_lab_test_work_sheet_tbl
      WHERE wk_test_sub_result_id = ?
    `;

  db_con.query(querySelect, [result_sheet_data_id], (error, results) => {
    if (error) {
      res.status(500).send(error);
    } else {
      if (results.length === 0) {
        // No data found for the specified lab test ID
        res.status(404).json({
          message: "No data found for the specified lab test worksheet ID",
        });
      } else {
        res.status(200).json(results[0]);
      }
    }
  });
};

// ! Update lab test worksheet data query
const updateLabTestWorksheetData = (req, res) => {
  const sub_result_id = req.params.id;
  const {
    wk_test_sub_result_name,
    wk_test_sub_topic,
    wk_test_units,
    wk_test_default_range,
    wk_test_default_value,
    wk_test_flug_range_min_female,
    wk_test_flug_range_max_female,
    wk_test_flug_range_min_male,
    wk_test_flug_range_max_male,
    wk_test_ref_range_topic_1,
    wk_test_ref_range_name_1,
    wk_test_ref_range_1,
    wk_test_ref_range_topic_2,
    wk_test_ref_range_name_2,
    wk_test_ref_range_2,
    wk_test_ref_range_topic_3,
    wk_test_ref_range_name_3,
    wk_test_ref_range_3,
    wk_test_ref_range_topic_4,
    wk_test_ref_range_name_4,
    wk_test_ref_range_4,
    wk_test_ref_range_topic_5,
    wk_test_ref_range_name_5,
    wk_test_ref_range_5,
    wk_test_ref_range_topic_6,
    wk_test_ref_range_name_6,
    wk_test_ref_range_6,
    wk_test_ref_range_topic_7,
    wk_test_ref_range_name_7,
    wk_test_ref_range_7,
    wk_test_ref_range_topic_8,
    wk_test_ref_range_name_8,
    wk_test_ref_range_8,
    wk_test_ref_range_topic_9,
    wk_test_ref_range_name_9,
    wk_test_ref_range_9,
    wk_test_ref_range_topic_10,
    wk_test_ref_range_name_10,
    wk_test_ref_range_10,
    wk_test_ref_range_topic_11,
    wk_test_ref_range_name_11,
    wk_test_ref_range_11,
    wk_test_ref_range_topic_12,
    wk_test_ref_range_name_12,
    wk_test_ref_range_12,
    wk_test_ref_range_topic_13,
    wk_test_ref_range_name_13,
    wk_test_ref_range_13,
    wk_test_ref_range_topic_14,
    wk_test_ref_range_name_14,
    wk_test_ref_range_14,
    wk_test_ref_range_show_status,
  } = req.body;

  // Update lab test details
  const queryUpdate = `
      UPDATE sys_lab_test_work_sheet_tbl 
      SET 
      wk_test_sub_result_name = ?,
      wk_test_sub_topic = ?,
      wk_test_units = ?,
      wk_test_default_range = ?,
      wk_test_default_value = ?,
      wk_test_flug_range_min_female = ?,
      wk_test_flug_range_max_female = ?,
      wk_test_flug_range_min_male = ?,
      wk_test_flug_range_max_male = ?,
      wk_test_ref_range_topic_1 = ?,
      wk_test_ref_range_name_1 = ?,
      wk_test_ref_range_1 = ?,
      wk_test_ref_range_topic_2 = ?,
      wk_test_ref_range_name_2 = ?,
      wk_test_ref_range_2 = ?,
      wk_test_ref_range_topic_3 = ?,
      wk_test_ref_range_name_3 = ?,
      wk_test_ref_range_3 = ?,
      wk_test_ref_range_topic_4 = ?,
      wk_test_ref_range_name_4 = ?,
      wk_test_ref_range_4 = ?,
      wk_test_ref_range_topic_5 = ?,
      wk_test_ref_range_name_5 = ?,
      wk_test_ref_range_5 = ?,
      wk_test_ref_range_topic_6 = ?,
      wk_test_ref_range_name_6 = ?,
      wk_test_ref_range_6 = ?,
      wk_test_ref_range_topic_7 = ?,
      wk_test_ref_range_name_7 = ?,
      wk_test_ref_range_7 = ?,
      wk_test_ref_range_topic_8 = ?,
      wk_test_ref_range_name_8 = ?,
      wk_test_ref_range_8 = ?,
      wk_test_ref_range_topic_9 = ?,
      wk_test_ref_range_name_9 = ?,
      wk_test_ref_range_9 = ?,
      wk_test_ref_range_topic_10 = ?,
      wk_test_ref_range_name_10 = ?,
      wk_test_ref_range_10 = ?,
      wk_test_ref_range_topic_11 = ?,
      wk_test_ref_range_name_11 = ?,
      wk_test_ref_range_11 = ?,
      wk_test_ref_range_topic_12 = ?,
      wk_test_ref_range_name_12 = ?,
      wk_test_ref_range_12 = ?,
      wk_test_ref_range_topic_13 = ?,
      wk_test_ref_range_name_13 = ?,
      wk_test_ref_range_13 = ?,
      wk_test_ref_range_topic_14 = ?,
      wk_test_ref_range_name_14 = ?,
      wk_test_ref_range_14 = ?,
      wk_test_ref_range_show_status = ?
      WHERE wk_test_sub_result_id = ?
    `;
  const values = [
    wk_test_sub_result_name,
    wk_test_sub_topic,
    wk_test_units,
    wk_test_default_range,
    wk_test_default_value,
    wk_test_flug_range_min_female,
    wk_test_flug_range_max_female,
    wk_test_flug_range_min_male,
    wk_test_flug_range_max_male,
    wk_test_ref_range_topic_1,
    wk_test_ref_range_name_1,
    wk_test_ref_range_1,
    wk_test_ref_range_topic_2,
    wk_test_ref_range_name_2,
    wk_test_ref_range_2,
    wk_test_ref_range_topic_3,
    wk_test_ref_range_name_3,
    wk_test_ref_range_3,
    wk_test_ref_range_topic_4,
    wk_test_ref_range_name_4,
    wk_test_ref_range_4,
    wk_test_ref_range_topic_5,
    wk_test_ref_range_name_5,
    wk_test_ref_range_5,
    wk_test_ref_range_topic_6,
    wk_test_ref_range_name_6,
    wk_test_ref_range_6,
    wk_test_ref_range_topic_7,
    wk_test_ref_range_name_7,
    wk_test_ref_range_7,
    wk_test_ref_range_topic_8,
    wk_test_ref_range_name_8,
    wk_test_ref_range_8,
    wk_test_ref_range_topic_9,
    wk_test_ref_range_name_9,
    wk_test_ref_range_9,
    wk_test_ref_range_topic_10,
    wk_test_ref_range_name_10,
    wk_test_ref_range_10,
    wk_test_ref_range_topic_11,
    wk_test_ref_range_name_11,
    wk_test_ref_range_11,
    wk_test_ref_range_topic_12,
    wk_test_ref_range_name_12,
    wk_test_ref_range_12,
    wk_test_ref_range_topic_13,
    wk_test_ref_range_name_13,
    wk_test_ref_range_13,
    wk_test_ref_range_topic_14,
    wk_test_ref_range_name_14,
    wk_test_ref_range_14,
    wk_test_ref_range_show_status,
    sub_result_id,
  ];

  db_con.query(queryUpdate, values, (error, results) => {
    if (error) {
      res.status(500).send(error);
    } else {
      if (results.affectedRows > 0) {
        res
          .status(200)
          .json({ message: "Lab test worksheet data updated successfully" });
      } else {
        res.status(404).json({ message: "Lab test worksheet data not found" });
      }
    }
  });
};

// ! add new lab test worksheet comment
const addNewLabTestWorkSheetComment = (req, res) => {
  const { wk_com_lab_test_id, wk_comment } = req.body;

  // Validate required fields
  if (!wk_com_lab_test_id || !wk_comment) {
    return res.status(400).json({ error: "All fields are required" });
  }

  // Check if the lab test ID exists
  const queryCheckLabTestId =
    "SELECT COUNT(*) AS count FROM sys_lab_test_tbl WHERE lab_id = ?";
  db_con.query(
    queryCheckLabTestId,
    [wk_com_lab_test_id],
    (checkLabTestIdError, checkLabTestIdResults) => {
      if (checkLabTestIdError) {
        return res.status(500).send(checkLabTestIdError);
      }

      const labTestIdExists = checkLabTestIdResults[0].count > 0;

      if (!labTestIdExists) {
        return res.status(400).json({ error: "Lab test ID does not exist" });
      }

      // Get the max from existing comments
      const queryMaxId =
        "SELECT MAX(CAST(SUBSTRING(wk_com_id,2) AS UNSIGNED)) AS max_id FROM sys_lab_test_work_sheet_comment_tbl";
      db_con.query(queryMaxId, (maxIdError, results) => {
        if (maxIdError) {
          return res.status(500).send(maxIdError);
        }

        const maxID = results[0].max_id || 0;
        const nextID = maxID + 1;

        // Format the next comment ID
        const wkComId = "N" + String(nextID).padStart(5, "0");

        // Insert comment data
        const queryInsert = `
        INSERT INTO sys_lab_test_work_sheet_comment_tbl (wk_com_lab_test_id, wk_com_id, wk_comment) 
        VALUES (?, ?, ?)
      `;
        const values = [wk_com_lab_test_id, wkComId, wk_comment];

        db_con.query(queryInsert, values, (insertError, insertResults) => {
          if (insertError) {
            res.status(500).send(insertError);
          } else {
            res.status(201).json({
              message: "Comment added successfully",
              wkComId: wkComId,
            });
          }
        });
      });
    }
  );
};

// ! Get lab test worksheet data by worksheet sub result id
const getLabTestWorksheetCommentDataByCommentId = (req, res) => {
  const comment_id = req.params.id;

  // Get lab test data query
  const querySelect = `
      SELECT 
        *
      FROM sys_lab_test_work_sheet_comment_tbl
      WHERE wk_com_id = ?
    `;

  db_con.query(querySelect, [comment_id], (error, results) => {
    if (error) {
      res.status(500).send(error);
    } else {
      if (results.length === 0) {
        // No data found for the specified lab test ID
        res.status(404).json({
          message: "No data found for the specified lab test worksheet ID",
        });
      } else {
        res.status(200).json(results[0]);
      }
    }
  });
};

// ? view lab test worksheet comments data by lab test id
const getAllLabTestWorksheetCommentsByLabTestId = async (req, res) => {
  try {
    const { draw, start, length, search, order, columns } = req.query;
    const lab_test_id = req.params.lab_test_id;
    const page = Math.ceil(start / length) + 1;
    const limit = +length;
    const offset = (page - 1) * limit;

    const orderColumnIndex = order[0].column;
    const orderDirection = order[0].dir;
    const orderColumn = columns[orderColumnIndex].data;

    const searchPattern = `%${search.value}%`;
    let searchWhere = "";
    const searchParams = [];

    if (search.value) {
      searchWhere = " WHERE ";
      searchWhere += " (wk_comment LIKE ? ) ";
      searchParams.push(searchPattern);
    }

    if (lab_test_id) {
      searchWhere += searchWhere ? " AND " : " WHERE ";
      searchWhere += `wk_com_lab_test_id = ?`;
      searchParams.push(lab_test_id);
    }

    const selectQuery = `
    SELECT
          *
    FROM sys_lab_test_work_sheet_comment_tbl
    ${searchWhere}
    ORDER BY ${orderColumn} ${orderDirection}
    LIMIT ?, ?
    `;

    const countQuery = `
    SELECT COUNT(wk_com_id) AS total 
    FROM sys_lab_test_work_sheet_comment_tbl
    ${searchWhere};
      `;

    db_con.query(
      selectQuery,
      [...searchParams, offset, limit],
      (error, results) => {
        if (error) {
          return res.status(500).send(error);
        }

        if (results.length === 0) {
          return res.status(200).json({
            draw: +draw,
            recordsTotal: 0,
            recordsFiltered: 0,
            data: [],
            message: "No data found in the table.",
          });
        }

        db_con.query(countQuery, searchParams, (countError, countResults) => {
          if (countError) {
            return res.status(500).send(countError);
          }

          const totalRecords = countResults[0].total;
          const response = {
            draw: +draw,
            recordsTotal: totalRecords,
            recordsFiltered: totalRecords,
            data: results,
          };

          res.status(200).json(response);
        });
      }
    );
  } catch (error) {
    res.status(500).send(error.message);
  }
};

// ! update lab test worksheet comment by comment id
const updateLabTestWorkSheetComment = (req, res) => {
  const wk_com_id = req.params.id;
  const { wk_comment } = req.body;

  // Validate required fields
  if (!wk_com_id || !wk_comment) {
    return res
      .status(400)
      .json({ error: "wk_com_id and wk_comment are required fields" });
  }

  // Update comment
  const queryUpdate = `
    UPDATE sys_lab_test_work_sheet_comment_tbl 
    SET wk_comment = ?
    WHERE wk_com_id = ?
  `;
  const values = [wk_comment, wk_com_id];

  db_con.query(queryUpdate, values, (updateError, updateResults) => {
    if (updateError) {
      res.status(500).send(updateError);
    } else {
      res.status(200).json({
        message: "Comment updated successfully",
        updatedCommentId: wk_com_id,
      });
    }
  });
};

module.exports = {
  createLabTest,
  updateLabTest,
  getAllLabTests,
  getLabTestById,
  getAllLabTestName,
  getLabTestNamesAndID,
  getAllLabTestByHospitalId,
  updateLabTestPrice,
  getLabTestNamesAndIDOwnLabTest,
  createLabTestWorkSheet,
  getAllLabTestWorksheetDataByLabTestId,
  getLabTestWorksheetDataBySubResultId,
  updateLabTestWorksheetData,
  addNewLabTestWorkSheetComment,
  getLabTestWorksheetCommentDataByCommentId,
  getAllLabTestWorksheetCommentsByLabTestId,
  updateLabTestWorkSheetComment,
};
