const db_con = require("../../../config/database-connection");

const currentDate = new Date();

const formattedDate = currentDate.toLocaleString("en-US", {
  year: "numeric",
  month: "2-digit",
  day: "2-digit",
});

// Extracting date components and formatting as "YYYY-MM-DD"
const [month, day, year] = formattedDate.split("/");
const Today_Date = `${year}-${month}-${day}`;

// ? create new empty result worksheet for lab test
const createNewLabTestEmptyResultSheet = (req, res) => {
  const { labTestId, refId, userId } = req.params;

  // Validate required fields
  if (!labTestId || !refId || !userId) {
    return res.status(400).json({ error: "All fields are required" });
  }

  // Query to select data from sys_lab_test_work_sheet_tbl
  const querySelect =
    "SELECT wk_test_sub_result_id, wk_test_default_value FROM sys_lab_test_work_sheet_tbl WHERE wk_lab_test_id = ?";

  db_con.query(querySelect, [labTestId], (selectError, selectResults) => {
    if (selectError) {
      return res.status(500).send(selectError);
    }

    // Check if there are any results to insert
    if (selectResults.length === 0) {
      return res
        .status(400)
        .json({ error: "No data found for the given lab test ID" });
    }

    const valuesToInsert = selectResults.map((result) => [
      refId,
      result.wk_test_sub_result_id,
      result.wk_test_default_value,
      new Date(),
      userId,
      userId,
    ]);

    // Query to insert data into sys_lab_test_result_tbl
    const queryInsert =
      "INSERT INTO sys_lab_test_result_tbl (res_refference_id, res_result_id, res_result_value, res_datetime, res_user, res_approve_user) VALUES ?";

    db_con.query(
      queryInsert,
      [valuesToInsert],
      (insertError, insertResults) => {
        if (insertError) {
          return res.status(500).send(insertError);
        } else {
          // Now, select data from sys_lab_test_work_sheet_comment_tbl
          const commentQuerySelect =
            "SELECT wk_com_lab_test_id, wk_comment FROM sys_lab_test_work_sheet_comment_tbl WHERE wk_com_lab_test_id = ?";

          db_con.query(
            commentQuerySelect,
            [labTestId],
            (commentSelectError, commentSelectResults) => {
              if (commentSelectError) {
                return res.status(500).send(commentSelectError);
              }

              const commentValuesToInsert = commentSelectResults.map(
                (commentResult) => [refId, commentResult.wk_comment]
              );

              // Query to insert data into sys_lab_test_result_comment_tbl
              const commentQueryInsert =
                "INSERT INTO sys_lab_test_result_comment_tbl (res_com_refference_id, res_com_comment) VALUES ?";

              db_con.query(
                commentQueryInsert,
                [commentValuesToInsert],
                (commentInsertError, commentInsertResults) => {
                  if (commentInsertError) {
                    return res.status(500).send(commentInsertError);
                  } else {
                    res.status(201).json({
                      message:
                        "Lab test results and comments added successfully",
                    });
                  }
                }
              );
            }
          );
        }
      }
    );
  });
};

module.exports = {
  createNewLabTestEmptyResultSheet,
};
