const express = require("express");
const router = express.Router();
const labTestController = require("./lab-test-controller");
const labTestResultSheetController = require("./lab-test-result-sheet-controller");

// * view all lab test worksheet data by latest id table view
router.get(
  "/view-all-lab-test-worksheet-data-by-lab-test-id/:lab_test_id",
  labTestController.getAllLabTestWorksheetDataByLabTestId
);

// * view lab test worksheet data by sub result id
router.get(
  "/view-lab-test-worksheet-data-by-sub-result-id/:id",
  labTestController.getLabTestWorksheetDataBySubResultId
);

// * view lab test worksheet comments data by comment id
router.get(
  "/view-lab-test-worksheet-comment-data-by-comment-id/:id",
  labTestController.getLabTestWorksheetCommentDataByCommentId
);

// * view all lab test worksheet comments by lab test id table view
router.get(
  "/view-lab-test-worksheet-comment-by-comment-lab-test-id/:lab_test_id",
  labTestController.getAllLabTestWorksheetCommentsByLabTestId
);

// * view all lab test by hospital id table view
router.get(
  "/view-all-lab-test-by-hospital-id/:hospital_id",
  labTestController.getAllLabTestByHospitalId
);

// * get all patient names and ids
router.get("/lab-test-names/", labTestController.getLabTestNamesAndID);

// * Get all lab test names
router.get("/view-all-test/", labTestController.getAllLabTestName);

// * Get all own lab test names and id
router.get(
  "/view-all-own-test-name-id",
  labTestController.getLabTestNamesAndIDOwnLabTest
);

// * Get all lab test data for table view
router.get("/", labTestController.getAllLabTests);

// * Get selected lab test data
router.get("/:id", labTestController.getLabTestById);

// * Create a new lab test worksheet
router.post(
  "/create-lab-test-result-worksheet",
  labTestController.createLabTestWorkSheet
);

// * add new lab test worksheet comment
router.post(
  "/add-new-lab-test-result-worksheet-comment",
  labTestController.addNewLabTestWorkSheetComment
);

// * Create a new lab test empty result sheet
router.post(
  "/create-new-empty-lab-test-result-sheet/:labTestId/:refId/:userId",
  labTestResultSheetController.createNewLabTestEmptyResultSheet
);

// * Create a new lab test
router.post("/", labTestController.createLabTest);

// * Update an existing lab test
router.put("/lab-test-price-update/:id", labTestController.updateLabTestPrice);

// * Update an lab test worksheet data
router.put(
  "/lab-test-worksheet-data-update/:id",
  labTestController.updateLabTestWorksheetData
);

// * Update an lab test worksheet comment data by comment id
router.put(
  "/lab-test-worksheet-comment-update/:id",
  labTestController.updateLabTestWorkSheetComment
);

// * Update an existing lab test
router.put("/:id", labTestController.updateLabTest);

module.exports = router;
