function generateBillNoGenId(dbConnection, branchId, typeId, callback) {
  // Retrieve the max ID for the given branch and type
  const query = `
        SELECT MAX(SUBSTRING(tra_bill_gen_no, 7)) AS maxId 
        FROM tmp_generate_pl_tra_bill_no_tbl 
        WHERE tra_bill_gen_branch = ? AND tra_bill_gen_type = ?`;

  dbConnection.query(query, [branchId, typeId], (error, results) => {
    if (error) {
      return callback(new Error(`Error fetching max ID: ${error.message}`));
    }

    let newId;
    if (results && results[0].maxId !== null) {
      const nextNumber = parseInt(results[0].maxId, 10) + 1;
      newId = `${branchId}-${typeId}${String(nextNumber).padStart(9, "0")}`;
    } else {
      newId = `${branchId}-${typeId}000000001`;
    }

    // Insert the newId into the database
    const insertQuery = `
          INSERT INTO tmp_generate_pl_tra_bill_no_tbl (tra_bill_gen_no, tra_bill_gen_branch, tra_bill_gen_type) 
          VALUES (?, ?, ?)`;

    dbConnection.query(
      insertQuery,
      [newId, branchId, typeId],
      (insertError) => {
        if (insertError) {
          return callback(
            new Error(`Error inserting new ID: ${insertError.message}`)
          );
        }
        callback(null, newId);
      }
    );
  });
}

module.exports = generateBillNoGenId;
